/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.helper;

import com.netease.nimlib.sdk.chatroom.ChatRoomMessageBuilder;
import com.netease.nimlib.sdk.chatroom.model.ChatRoomMessage;
import com.netease.nimlib.sdk.msg.constant.MsgTypeEnum;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.util.Map;

import im.netease.nim.msg.CustomAttachment;

/**
 * Created by hzsunyj on 2019-12-25.
 */
public class ChatRoomMessageHelper {

    public static ChatRoomMessage convert(Map map) {
        String roomId = (String) map.get("sessionId");
        String attach = (String) map.get("attach");
        int msgType = (int) map.get("msgType");
        String content = (String) map.get("content");
        if (msgType == MsgTypeEnum.text.getValue()) {
            return ChatRoomMessageBuilder.createChatRoomTextMessage(roomId, content);
        } else if (msgType == MsgTypeEnum.custom.getValue()) {
            return ChatRoomMessageBuilder.createChatRoomCustomMessage(roomId, new CustomAttachment(attach));
        } else if (isFileAware(msgType)) {
            try {
                JSONObject jsonObject = new JSONObject(attach);
                String path = jsonObject.optString("path");
                if (path == null) {
                    return null;
                }
                File file = new File(path);
                if (file == null || !file.exists()) {
                    return null;
                }
                String displayName = jsonObject.optString("name");
                String sen = jsonObject.optString("sen");
                if (msgType == MsgTypeEnum.image.getValue()) {
                    return ChatRoomMessageBuilder.createChatRoomImageMessage(roomId, new File(path), displayName, sen);
                } else if (msgType == MsgTypeEnum.file.getValue()) {
                    return ChatRoomMessageBuilder.createChatRoomFileMessage(roomId, new File(path), displayName, sen);
                }
            } catch (JSONException e) {
                return null;
            }
        }
        return null;
    }

    public static boolean isFileAware(int type) {
        return (type == MsgTypeEnum.image.getValue()) || (type == MsgTypeEnum.file.getValue());
    }

}
