/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.msg;

import android.content.Context;
import android.text.TextUtils;

import com.netease.nimlib.sdk.NIMClient;
import com.netease.nimlib.sdk.Observer;
import com.netease.nimlib.sdk.RequestCallbackWrapper;
import com.netease.nimlib.sdk.ResponseCode;
import com.netease.nimlib.sdk.msg.MsgService;
import com.netease.nimlib.sdk.msg.MsgServiceObserve;
import com.netease.nimlib.sdk.msg.model.IMMessage;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import im.netease.nim.base.Handler;
import im.netease.nim.helper.MessageHelper;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by hzsunyj on 2019-12-25.
 */
public class MsgBridge extends Handler {

    public MsgBridge(MethodChannel channel, Context context) {
        super(channel, context);
    }

    MsgService msgService;

    MsgServiceObserve msgServiceObserve;

    Observer<List<IMMessage>> observer = new Observer<List<IMMessage>>() {

        @Override
        public void onEvent(List<IMMessage> imMessages) {
            onReceiveMessage(imMessages);
        }
    };

    @Override
    public void unInit() {
        if (msgServiceObserve != null) {
            msgServiceObserve.observeReceiveMessage(observer, false);
        }
        msgService = null;
        msgServiceObserve = null;
    }

    @Override
    public void postInit() {
        msgService = NIMClient.getService(MsgService.class);
        msgServiceObserve = NIMClient.getService(MsgServiceObserve.class);
    }

    @Override
    public void registerObserver() {
        msgServiceObserve.observeReceiveMessage(observer, true);
    }

    private void onReceiveMessage(List<IMMessage> imMessages) {
        MapBuilder builder = buildArg(observerModuleName());
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>(imMessages.size());
        for (IMMessage message : imMessages) {
            if (message == null) {
                continue;
            }
            MapBuilder data = map();
            buildMessage(message, data);
            arrayList.add(data.build());
        }
        builder.put("message", arrayList);
        channel.invokeMethod("onReceiveMessage", builder.build());
    }

    private void buildMessage(IMMessage message, MapBuilder data) {
        data.put("uuid", message.getUuid());
        data.put("sessionId", message.getSessionId());
        data.put("sessionType", message.getSessionType().getValue());
        data.put("attach", message.getAttachStr());
        data.put("time", message.getTime());
        data.put("serverId", message.getServerId());
        data.put("msgType", message.getMsgType().getValue());
        data.put("content", message.getContent());
        data.put("fromAccount", message.getFromAccount());
        data.put("msgStatus", message.getStatus().getValue());
        data.put("msgDirection", message.getDirect().getValue());
    }

    @Override
    public String moduleName() {
        return "MsgService";
    }
    @Override
    public String observerModuleName() {
        return "MsgObserver";
    }

    @Override
    public void handle(String method, MethodCall call, MethodChannel.Result result) {
        if (TextUtils.isEmpty(method)) {
            return;
        }
        switch (method) {
            case "sendMessage":
                onSendMessage(call);
                break;
        }
        result.success(null);
    }

    private void onSendMessage(final MethodCall call) {
        Map map = call.argument("message");
        if (map == null) {
            MapBuilder builder = buildArg(getReceiptId(call), moduleName());
            builder.put("code", ResponseCode.RES_EPARAM);
            channel.invokeMethod("onSendMessage", builder.build());
        } else {
            boolean resend = call.argument("resend");
            IMMessage message = MessageHelper.convert(map);
            if (message == null) {
                MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                builder.put("code", ResponseCode.RES_EPARAM);
                channel.invokeMethod("onSendMessage", builder.build());
                return;
            }
            msgService.sendMessage(message, resend).setCallback(new RequestCallbackWrapper<Void>() {

                @Override
                public void onResult(int i, Void aVoid, Throwable throwable) {
                    MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                    builder.put("code", i);
                    channel.invokeMethod("onSendMessage", builder.build());
                }
            });
        }
    }
}
