/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

package im.netease.nim.passthrough;

import android.content.Context;
import android.text.TextUtils;

import com.netease.nimlib.sdk.NIMClient;
import com.netease.nimlib.sdk.Observer;
import com.netease.nimlib.sdk.RequestCallbackWrapper;
import com.netease.nimlib.sdk.passthrough.PassthroughService;
import com.netease.nimlib.sdk.passthrough.PassthroughServiceObserve;
import com.netease.nimlib.sdk.passthrough.model.PassthroughNotifyData;
import com.netease.nimlib.sdk.passthrough.model.PassthroughProxyData;

import im.netease.nim.base.Handler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Created by hzsunyj on 2019-12-24.
 */
public class PassthroughBridge extends Handler {

    public PassthroughBridge(MethodChannel channel, Context context) {
        super(channel, context);
    }

    private PassthroughService passthroughService;

    private PassthroughServiceObserve passthroughServiceObserve;


    Observer<PassthroughNotifyData> notifyDataObserver = new Observer<PassthroughNotifyData>() {

        @Override
        public void onEvent(PassthroughNotifyData data) {
            MapBuilder builder = buildArg(observerModuleName());
            builder.put("fromAccid", data.getFromAccid());
            builder.put("body", data.getBody());
            builder.put("time", data.getTime());
            channel.invokeMethod("onPassthroughNotify", builder.build());
        }
    };

    @Override
    public void unInit() {
        if (passthroughServiceObserve != null) {
            passthroughServiceObserve.observePassthroughNotify(notifyDataObserver, false);
        }
        passthroughServiceObserve = null;
        passthroughService = null;
    }

    @Override
    public void postInit() {
        passthroughService = NIMClient.getService(PassthroughService.class);
        passthroughServiceObserve = NIMClient.getService(PassthroughServiceObserve.class);
    }

    @Override
    public void registerObserver() {
        passthroughServiceObserve.observePassthroughNotify(notifyDataObserver, true);
    }

    @Override
    public String moduleName() {
        return "PassthroughService";
    }
    @Override
    public String observerModuleName() {
        return "PassthroughObserver";
    }
    @Override
    public void handle(String method, MethodCall call, MethodChannel.Result result) {
        if (TextUtils.isEmpty(method)) {
            return;
        }
        switch (method) {
            case "httpProxy":
                onHttpProxy(call);
                break;
        }
        result.success(null);
    }

    private void onHttpProxy(final MethodCall call) {
        String zone = call.argument("zone");
        String path = call.argument("path");
        Integer method1 = call.argument("method");
        int method = method1 == null ? 2 : method1;
        String header = call.argument("header");
        String body = call.argument("body");
        passthroughService.httpProxy(new PassthroughProxyData(zone, path, method, header, body)).setCallback(
                new RequestCallbackWrapper<PassthroughProxyData>() {

                    @Override
                    public void onResult(int i, PassthroughProxyData proxyData, Throwable throwable) {
                        MapBuilder builder = buildArg(getReceiptId(call), moduleName());
                        builder.put("code", i);
                        if (i == 200) {
                            builder.put("zone", proxyData.getZone());
                            builder.put("path", proxyData.getPath());
                            builder.put("method", proxyData.getMethod());
                            builder.put("header", proxyData.getHeader());
                            builder.put("body", proxyData.getBody());
                        }
                        channel.invokeMethod("onHttpProxy", builder.build());
                    }
                });
    }
}
