// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'dart:async';

import 'package:nim/nim.dart';
import 'package:nim_example/main/mainpage.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:nim_example/util//toast.dart';

class LoginPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return _LoginPageState();
  }
}

class _LoginPageState extends State<LoginPage> {
  final GlobalKey<FormState> formKey = GlobalKey<FormState>();
 String appKey;
  TextEditingController _accountController;
  TextEditingController _passController;

  @override
  void initState() {
    super.initState();
    requestPermission(context);
    registerOnlineStatus();
    appKey = "6f6a8e7a18d7b43adfd7f17ad3da3c8e"; //测试使用APPkey
    _accountController = new TextEditingController(text: "请输入用户ID");
    _passController = new TextEditingController(text: "请输入密码");
  }

  void registerOnlineStatus() {
    NimPlugin().getAuthObserver().registerOnlineStatus((StatusCode code) {
      if (code == StatusCode.logined) {
        Navigator.of(context).push(
          MaterialPageRoute(builder: (BuildContext context) {
            return MainPage();
          }),
        );
      }
    });
  }

  Future requestPermission(BuildContext _context) async {
    PermissionStatus permission = await PermissionHandler().checkPermissionStatus(PermissionGroup.storage);
    if (permission != PermissionStatus.granted) {
      var group = await PermissionHandler().requestPermissions([PermissionGroup.storage]);
      var storagePermission = group[PermissionGroup.storage];
      if (storagePermission != PermissionStatus.granted) {
        ToastUtil.showToast(_context, "storage permission not granted");
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Login"),
      ),
      body: Container(
        padding: const EdgeInsets.fromLTRB(20, 60.0, 20, 0),
        child: Form(
            key: formKey,
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.stretch,
              children: buildInputs() + buildSubmitButtons(),
            )),
      ),
    );
  }

  bool validateAndSave() {
    final FormState form = formKey.currentState;
    if (form.validate()) {
      form.save();
      return true;
    }
    return false;
  }

  void login() {
    if (validateAndSave()) {
      var authInfo = AuthInfo(_accountController.text, _passController.text, appKey);
      NimPlugin().getAuthService().login(authInfo, (code,v) {
        print(code);
        print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
      });
    } else {
      print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
    }
  }

  List<Widget> buildInputs() {
    return <Widget>[
      TextFormField(
        key: Key('account'),
        decoration: InputDecoration(labelText: 'account'),
        controller: _accountController,
      ),
      Padding(
        padding: EdgeInsets.symmetric(vertical: 5.0),
      ),
      TextFormField(
        key: Key('password'),
        decoration: InputDecoration(labelText: 'password'),
        obscureText: true,
        controller: _passController,
      )
    ];
  }

  List<Widget> buildSubmitButtons() {
    return <Widget>[
      Padding(
        padding: EdgeInsets.symmetric(vertical: 10.0),
      ),
      Material(
        child: ElevatedButton(
          style: ButtonStyle(
              backgroundColor: MaterialStateProperty.resolveWith<Color>((states) {
                if (states.contains(MaterialState.disabled)) {
                  return Colors.redAccent;
                }
                return Colors.blue;
              }),
              padding: MaterialStateProperty.all(EdgeInsets.symmetric(vertical: 10))),
          key: Key('login'),
          child: Text('Login',
              style: TextStyle(
                fontSize: 20.0,
                color: Colors.white,
              )),
          onPressed: login,
        ),
        borderRadius: BorderRadius.circular(12),
      ),
      TextButton(
        style: ButtonStyle(
            padding: MaterialStateProperty.all(EdgeInsets.fromLTRB(0, 5, 0, 0))),
        child: Text(
          "login to nim",
          style: TextStyle(fontSize: 16.0, color: Colors.black45),
        ),
        onPressed: null,
      )
    ];
  }

}
