// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:nim/nim.dart';
import 'package:nim_example/util/toast.dart';

class ChatRoomPage extends StatefulWidget {
  const ChatRoomPage({Key key}) : super(key: key);

  State createState() {
    return _ChatRoomPageState();
  }
}

class _ChatRoomPageState extends State {

  String chatRoomInfo = "";//chatroom 信息记录
  final FocusNode _focusNode = FocusNode();
  TextEditingController _contentController;
  ChatRoomService chatRoomService;
  String _sessionID = "you session id";


  @override
  void initState() {
    super.initState();
    _contentController = TextEditingController();
    chatRoomService = NimPlugin().getChatRoomService();
    //接收消息
    // NimPlugin().getChatRoomObserver().registerReceiveMessage();
  }

  void sendChatRoomMsg(){
    IMMessage message = IMMessage();
    String text = _contentController.text;
    if(text == null){
      ToastUtil.showToast(context, "msg can not be empty");
      return;
    }
    message.content = text;
    message.sessionId = _sessionID;
    message.sessionType = SessionType.p2p;
    message.msgType = MsgType.text;
    // NimPlugin().getChatRoomService().sendMessage(message: message, callback: (code, o) {
    //   if(code == 200){
    //     ToastUtil.showToast(context, "msg send success ${text}");
    //   }
    // });
  }

  void handlerRecChatRoomMsg(List<IMMessage> msgs){
    for(IMMessage msg in msgs){
      ToastUtil.showToast(context, "Receiver msg:${msg.content}");
    }
  }

  void enterChatRoom(){
    // NimPlugin().getChatRoomService().sendMessage(message, resend, (code, data) { })
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Color(0xfff6f6f6),
        body: buildBody(),
      );
  }

  Widget buildBody() {
    return Scaffold(
      appBar: AppBar(
        title: Text('聊天室'),
      ),
      body: Column(
        children: [buildChatRoomBody(),SizedBox(height: 10)],
      ),

    );
  }

  Widget buildChatRoomBody() {
    return Container(
      color: Colors.white,
      height: 200,
      padding: EdgeInsets.only(left: 10, top: 8, bottom: 8, right: 10),
      child: Expanded(
          child: Column(
            children: <Widget>[ shadow("Message"),  Row(

              children: <Widget>[buildInputBox("messageInputText","input message"), buildSend("messageSendBtn","sendMsg",sendChatRoomMsg)],
            )],
          )),
    );
  }

  Widget buildOperateBar(){
    return Row(children:  [TextButton(child: Text(""))],);
  }

  Widget shadow(String title) {
    return Container(
      height: 40,
      child: Text(title),
      alignment: Alignment.center,
      decoration: BoxDecoration(
        color: Color(0xffffffff),
        boxShadow: [
          BoxShadow(
            color:Color(0xffe1e3e6),
          ),
        ],
      ),
    );
  }

  Widget buildInputBox(String txKey,String hitMsg) {
    return Expanded(
        child: Container(
          decoration: BoxDecoration(
              color: Color(0xfff7f8fa),
              borderRadius: BorderRadius.all(Radius.circular(20)),
              border: Border.all(width: 1, color: Color(0xfff2f3f5))),
          height: 40,
          alignment: Alignment.center,
          child: TextField(
              key: ValueKey(txKey),
              focusNode: _focusNode,
              controller: _contentController,
              cursorColor: Color(0xff337eff),
              keyboardAppearance: Brightness.light,
              textAlignVertical: TextAlignVertical.center,
              decoration: InputDecoration(
                filled: true,
                isDense: true,
                fillColor: Colors.transparent,
                hintText: hitMsg,
                hintStyle: TextStyle(
                    fontSize: 14,
                    color: Color(0xff999999),
                    decoration: TextDecoration.none),
                border: InputBorder.none,
              )),
        ));
  }

  Widget buildSend(String valueKey,String text,Function click) {
    return Container(
      margin: EdgeInsets.only(left: 8),
      height: 28,
      width: 52,
      child: ElevatedButton(
        key: ValueKey(valueKey),
        style: ButtonStyle(
            backgroundColor: MaterialStateProperty.resolveWith<Color>((states) {
              if (states.contains(MaterialState.disabled)) {
                return Color(0x7f337eff);
              }
              return Color(0x7f337eff);
            }),
            padding:
            MaterialStateProperty.all(EdgeInsets.symmetric(vertical: 0)),
            shape: MaterialStateProperty.all(RoundedRectangleBorder(
                side: BorderSide(color: Color(0x7f337eff)),
                borderRadius: BorderRadius.all(Radius.circular(14))))),
        onPressed: click,
        child: Text(
          text,
          style: TextStyle(color: Colors.white, fontSize: 12),
          textAlign: TextAlign.center,
        ),
      ),
    );
  }
}
