// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:nim/nim.dart';
import 'package:nim_example/util/toast.dart';

class FriendsPage extends StatefulWidget {
  const FriendsPage({Key key}) : super(key: key);

  State createState() {
    return _FriendsPageState();
  }
}

class _FriendsPageState extends State {
  String opResult = '';
  TextEditingController _searchController;
  TextEditingController _requestUserController;

  @override
  void initState() {
    super.initState();
    _searchController = TextEditingController();
    _requestUserController = TextEditingController();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("添加好友"),
      ),
      body: Column(
        children: [
          SizedBox(
            height: 8,
          ),
          Row(children: [
            buildInputBox(
                "searchFriendsIdInput", "请输入搜索的用户ID", _searchController),
            buildButton(
                "searchFriendsButton", "搜索", _searchController, _onSearch)
          ]),
          SizedBox(
            height: 8,
          ),
          Row(children: [
            buildInputBox(
                "requestFriendsIdInput", "请输入添加好友的ID", _requestUserController),
            buildButton("requestFriendsButton", "添加好友", _requestUserController,
                requestFriends)
          ]),
          SizedBox(
            height: 8,
          ),
          Row(children: [
            buildButton("searchFriendsButton", "获取好友列表", null, getMyFriendsList)
          ]),
          Text(
            opResult,
            maxLines: 20,
            style: TextStyle(
                fontSize: 16.0,
                fontWeight: FontWeight.bold,
                color: Colors.black),
          ),
        ],
      ),
      backgroundColor: Color(0xffe2e2e2),
    );
  }

  ///search user base on userId
  void _onSearch(String userId) {
    NimPlugin().getFriendService().fetchUsers([userId], (code, friends) {
      if (code == 200) {
        ToastUtil.showToast(context, "search user success");
        if (friends != null && friends is List<User>) {
          List<User> usersList = friends;
          StringBuffer stringBuffer = StringBuffer();
          usersList.forEach((element) {
            stringBuffer
                .writeln(element.userId + ':' + element.userInfo.nickName);
          });
          setState(() {
            opResult = "Search Result:" + stringBuffer.toString();
          });
        }
      } else {
        ToastUtil.showToast(context, "search user fail $code");
      }
    });
  }

  ///request to add new friend
  void requestFriends(String userId) {
    FriendRequest friendRequest =
        FriendRequest(userId, FriendOperation.add, "I want you");
    NimPlugin().getFriendService().requestFriend(friendRequest, (code, data) {
      if (code == 200) {
        ToastUtil.showToast(context, "request user success");
        setState(() {
          opResult = "request to add user $userId success";
        });
      } else {
        ToastUtil.showToast(context, "request user fail $code");
      }
    });
  }

  ///get my friends list
  void getMyFriendsList(String info) {
    NimPlugin().getFriendService().myFriends().then((value) {
      if(value != null){
        StringBuffer stringBuffer = StringBuffer();
        value.forEach((element) {
          stringBuffer
              .writeln(element.userId + ':' + element.userInfo.nickName);
        });
        setState(() {
          opResult = "Search Result:" + stringBuffer.toString();
        });
      }
    });
  }

  Widget buildInputBox(
      String txKey, String hitMsg, TextEditingController controller) {
    return Expanded(
      child: Container(
        margin: EdgeInsets.only(left: 10, right: 10),
        decoration: BoxDecoration(
            color: Color(0xfff7f8fa),
            borderRadius: BorderRadius.all(Radius.circular(20)),
            border: Border.all(width: 1, color: Color(0xfff2f3f5))),
        height: 40,
        alignment: Alignment.center,
        child: TextField(
            key: ValueKey(txKey),
            controller: controller,
            cursorColor: Color(0xff337eff),
            keyboardAppearance: Brightness.light,
            textAlignVertical: TextAlignVertical.center,
            decoration: InputDecoration(
              filled: true,
              isDense: true,
              fillColor: Colors.transparent,
              hintText: hitMsg,
              hintStyle: TextStyle(
                  fontSize: 14,
                  color: Color(0xff999999),
                  decoration: TextDecoration.none),
              border: InputBorder.none,
            )),
      ),
      flex: 2,
    );
  }

  Widget buildButton(String valueKey, String text,
      TextEditingController controller, Function click) {
    return Container(
      margin: EdgeInsets.only(left: 10, right: 10),
      height: 28,
      width: 82,
      child: ElevatedButton(
        key: ValueKey(valueKey),
        style: ButtonStyle(
            backgroundColor: MaterialStateProperty.resolveWith<Color>((states) {
              if (states.contains(MaterialState.disabled)) {
                return Color(0x7f337eff);
              }
              return Color(0x7f337eff);
            }),
            padding:
                MaterialStateProperty.all(EdgeInsets.symmetric(vertical: 0)),
            shape: MaterialStateProperty.all(RoundedRectangleBorder(
                side: BorderSide(color: Color(0x7f337eff)),
                borderRadius: BorderRadius.all(Radius.circular(14))))),
        onPressed: click(controller?.text),
        child: Text(
          text,
          style: TextStyle(color: Colors.white, fontSize: 12),
          textAlign: TextAlign.center,
        ),
      ),
    );
  }
}
