// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:nim/nim.dart';
import 'package:nim_example/util/toast.dart';

class ChatPage extends StatefulWidget {
  const ChatPage({Key key}) : super(key: key);

  State createState() {
    return _ChatPageState();
  }
}

class _ChatPageState extends State {
  final FocusNode _focusNode = FocusNode();
  AuthInfo _authInfo;
  String opResult = "操作结果展示：";
  String _sessionID = "you session id";
  TextEditingController _sendTextController;
  TextEditingController _chatRoomIdTextController;
  TextEditingController _chatRoomNickTextController;

  @override
  void initState() {
    super.initState();
    _sendTextController = TextEditingController();
    _chatRoomIdTextController = TextEditingController();
    _chatRoomNickTextController = TextEditingController();
    NimPlugin().getAuthService().getLoginInfo().then((value) => {
      _authInfo = value
    });
    //接收消息
    NimPlugin().getMsgObserver().registerReceiveMessage(handlerRecMsg);
  }

  ///进入聊天室
  void enterChatRoom(){
    String chatRoomId = _chatRoomIdTextController.text;
    String chatRoomNick = _chatRoomNickTextController.text;
    if(chatRoomId == null){
      ToastUtil.showToast(context, "please input chatRoom id");
      return;
    }
    EnterChatRoomData data = EnterChatRoomData();
    data.account = _authInfo?.account;
    data.appKey = _authInfo?.appKey;
    data.roomId = chatRoomId;
    data.nick = chatRoomNick;
    ///进入聊天室回调，返回聊天室信息
    NimPlugin().getChatRoomService().enterChatRoom(data, 3, (code, data) {
      if(code == 200){
        setState(() {
          opResult += ",enter room success:" + data.roomId;
        });
      }else{
        ToastUtil.showToast(context, "enter error:$code");
      }
    });
  }

  ///发送聊天室消息
  void sendMsg(){
    ChatRoomMessage message = ChatRoomMessage();
    String text = _sendTextController.text;
    if(text == null){
      ToastUtil.showToast(context, "msg can not be empty");
      return;
    }
    message.content = text;
    message.sessionId = _sessionID;
    message.sessionType = SessionType.p2p;
    message.msgType = MsgType.text;
    NimPlugin().getChatRoomService().sendMessage(message, false, (code, data) {
      if(code == 200){
        ToastUtil.showToast(context, "msg send success $text");
        setState(() {
          opResult += ",send msg success:" + text;
        });
    }else{
        ToastUtil.showToast(context, "msg send fail $text");
      }});

  }

  void handlerRecMsg(List<IMMessage> msgs){
    for(IMMessage msg in msgs){
      ToastUtil.showToast(context, "Receiver msg:${msg.content}");
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('聊天室'),
      ),
      backgroundColor: Color(0xfff6f6f6),
      body: buildChatBody(),
    );
  }

  Widget buildChatBody() {
    return Container(
      color: Colors.white,
      height: 400,
      padding: EdgeInsets.only(left: 10, top: 8, bottom: 8, right: 10),
      child: Expanded(
          child: Column(
            children: <Widget>[
              shadow("进入聊天室"),
              buildInputBox("chatRoomIdInputText","请输入聊天室ID",_chatRoomIdTextController),
              buildInputBox("chatRoomNickNameInputText","昵称",_chatRoomNickTextController),
              buildSend("enterChatRoomBtn","进入房间",enterChatRoom),
              shadow("发送消息"),
              Row(
                children: <Widget>[Expanded(child: buildInputBox("messageInputText","input message",_sendTextController)), buildSend("messageSendBtn","sendMsg",sendMsg)],
              ),
              shadow(opResult) ,
            ],

          )),
    );
  }

  Widget shadow(String title) {
    return Container(
      height: 60,
      child: Text(title),
      alignment: Alignment.center,
      decoration: BoxDecoration(
        color: Color(0xffffffff),
        boxShadow: [
          BoxShadow(
            color:Color(0xffe1e3e6),
          ),
        ],
      ),
    );
  }

  Widget buildInputBox(String txKey,String hitMsg,TextEditingController controller) {
    return  Container(
          decoration: BoxDecoration(
              color: Color(0xfff7f8fa),
              borderRadius: BorderRadius.all(Radius.circular(20)),
              border: Border.all(width: 1, color: Color(0xfff2f3f5))),
          height: 40,
          alignment: Alignment.center,
          child: TextField(
              key: ValueKey(txKey),
              // focusNode: _focusNode,
              controller: controller,
              cursorColor: Color(0xff337eff),
              keyboardAppearance: Brightness.light,
              textAlignVertical: TextAlignVertical.center,
              decoration: InputDecoration(
                filled: true,
                isDense: true,
                fillColor: Colors.transparent,
                hintText: hitMsg,
                hintStyle: TextStyle(
                    fontSize: 14,
                    color: Color(0xff999999),
                    decoration: TextDecoration.none),
                border: InputBorder.none,
              )),
        );
  }

  Widget buildSend(String valueKey,String text,Function click) {
    return Container(
      margin: EdgeInsets.only(left: 8),
      height: 28,
      width: 60,
      child: ElevatedButton(
        key: ValueKey(valueKey),
        style: ButtonStyle(
            backgroundColor: MaterialStateProperty.resolveWith<Color>((states) {
              if (states.contains(MaterialState.disabled)) {
                return Color(0x7f337eff);
              }
              return Color(0x7f337eff);
            }),
            padding:
            MaterialStateProperty.all(EdgeInsets.symmetric(vertical: 0)),
            shape: MaterialStateProperty.all(RoundedRectangleBorder(
                side: BorderSide(color: Color(0x7f337eff)),
                borderRadius: BorderRadius.all(Radius.circular(14))))),
        onPressed: click,
        child: Text(
          text,
          style: TextStyle(color: Colors.white, fontSize: 12),
          textAlign: TextAlign.center,
        ),
      ),
    );
  }
}
