// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:nim/nim.dart';
import 'package:nim_example/util/toast.dart';

class MessagePage extends StatefulWidget {
  const MessagePage({Key key}) : super(key: key);

  State createState() {
    return _MessagePageState();
  }
}

class _MessagePageState extends State {
  SignallingService service;
  final FocusNode _focusNode = FocusNode();
  TextEditingController _contentController;
  String _sessionID = "you session id";

  @override
  void initState() {
    super.initState();
    service = NimPlugin().getSignallingService();

    ///监听收到的message信息
    NimPlugin().getMsgObserver().registerReceiveMessage((msgs) {
      for (IMMessage msg in msgs) {
        ToastUtil.showToast(context, "Receiver msg:${msg.content}");
      }
    });
  }

  void createChannel() {
    service.create(
        type: ChannelType.custom,
        channelName: "xxxxxxxxxxxx",
        callback: (int code, ChannelBaseInfo info) {
          print("code==$code");
        });
  }

  ///发送message给指定用户，传入sessionId，sessionType发送消息的场景包括个人对个人，聊天室等
  void sendMsg() {
    IMMessage message = IMMessage();
    message.content = 'this is an example';
    message.sessionId = _sessionID;
    message.sessionType = SessionType.p2p;
    message.msgType = MsgType.text;
    NimPlugin().getMsgService().sendMessage(
        message: message,
        callback: (code, o) {
          if (code == 200) {
            ToastUtil.showToast(context, "msg send success");
          }
        });
  }

  void handlerRecMsg() {}

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('消息'),
      ),
      backgroundColor: Color(0xfff6f6f6),
      body: buildBody(),
    );
  }

  Widget buildBody() {
    return Container(
      color: Colors.white,
      child: buildNewBody(),
    );
  }

  Widget buildNewBody() {
    return Row(
      verticalDirection: VerticalDirection.down,
      children: [buildMsgBody()],
    );
  }

  Widget buildMsgBody() {
    return Container(
      color: Colors.white,
      alignment: Alignment.center,
      width: 400,
      height: 200,
      child: Expanded(
          child: Column(
        children: <Widget>[
          shadow("Message"),
          Row(
            children: <Widget>[
              buildInputBox("chatRoomInputText", "input message"),
              buildSend()
            ],
          )
        ],
      )),
    );
  }

  Widget shadow(String title) {
    return Container(
      height: 40,
      child: Text(title),
      alignment: Alignment.center,
      decoration: BoxDecoration(
        color: Color(0xffffffff),
        boxShadow: [
          BoxShadow(
            color: Color(0xffe1e3e6),
          ),
        ],
      ),
    );
  }

  Widget buildInputBox(String txKey, String hitMsg) {
    return Expanded(
        child: Container(
      decoration: BoxDecoration(
          color: Color(0xfff7f8fa),
          borderRadius: BorderRadius.all(Radius.circular(20)),
          border: Border.all(width: 1, color: Color(0xfff2f3f5))),
      height: 40,
      alignment: Alignment.center,
      child: TextField(
          key: ValueKey(txKey),
          focusNode: _focusNode,
          controller: _contentController,
          cursorColor: Color(0xff337eff),
          keyboardAppearance: Brightness.light,
          textAlignVertical: TextAlignVertical.center,
          decoration: InputDecoration(
            filled: true,
            isDense: true,
            fillColor: Colors.transparent,
            hintText: hitMsg,
            hintStyle: TextStyle(
                fontSize: 14,
                color: Color(0xff999999),
                decoration: TextDecoration.none),
            border: InputBorder.none,
          )),
    ));
  }

  Widget buildSend() {
    return Container(
      margin: EdgeInsets.only(left: 8),
      height: 28,
      width: 52,
      child: ElevatedButton(
        key: ValueKey("chatRoomSendBtn"),
        style: ButtonStyle(
            backgroundColor: MaterialStateProperty.resolveWith<Color>((states) {
              if (states.contains(MaterialState.disabled)) {
                return Color(0x7f337eff);
              }
              return Color(0x7f337eff);
            }),
            padding:
                MaterialStateProperty.all(EdgeInsets.symmetric(vertical: 0)),
            shape: MaterialStateProperty.all(RoundedRectangleBorder(
                side: BorderSide(color: Color(0x7f337eff)),
                borderRadius: BorderRadius.all(Radius.circular(14))))),
        onPressed: sendMsg,
        child: Text(
          "send",
          style: TextStyle(color: Colors.white, fontSize: 12),
          textAlign: TextAlign.center,
        ),
      ),
    );
  }
}
