// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:flutter/material.dart';
import 'package:nim_example/util/toast.dart';
import 'package:nim/nim.dart';

class PassThroughPage extends StatefulWidget {
  const PassThroughPage({Key key}) : super(key: key);

  State createState() {
    return _PassThroughPageState();
  }
}

class _PassThroughPageState extends State {
  String opResult = '';
  TextEditingController inputController;

  @override
  void initState() {
    super.initState();
    inputController = TextEditingController();
  }

  void sendPassThrough() {
    String text = inputController.text;
    if (text == null) {
      ToastUtil.showToast(context, text);
      return;
    }

    /// PassthroughProxyData可设置域名、地址、请求方式等
    PassthroughProxyData data = PassthroughProxyData();
    data.body = text;
    NimPlugin().getPassthroughService().httpProxy(data, (code, response) {
      if (code == 200) {
        ToastUtil.showToast(context, "send http success");
        setState(() {
          opResult = "返回数据：" + response.body;
        });
      }else{
        ToastUtil.showToast(context, "send http fail $code");
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("HTTP通道"),
      ),
      body: Column(
        children: [
          SizedBox(
            height: 8,
          ),
          Row(children: [
            Expanded(
              child: Container(
                margin: EdgeInsets.only(left: 10, right: 10),
                decoration: BoxDecoration(
                    color: Color(0xfff7f8fa),
                    borderRadius: BorderRadius.all(Radius.circular(20)),
                    border: Border.all(width: 1, color: Color(0xfff2f3f5))),
                height: 40,
                alignment: Alignment.center,
                child: TextField(
                    key: ValueKey('pathThroughTextField'),
                    controller: inputController,
                    cursorColor: Color(0xff337eff),
                    keyboardAppearance: Brightness.light,
                    textAlignVertical: TextAlignVertical.center,
                    decoration: InputDecoration(
                      filled: true,
                      isDense: true,
                      fillColor: Colors.transparent,
                      hintText: "请输入传输的内容",
                      hintStyle: TextStyle(
                          fontSize: 14,
                          color: Color(0xff999999),
                          decoration: TextDecoration.none),
                      border: InputBorder.none,
                    )),
              ),
              flex: 2,
            ),
            Container(
              margin: EdgeInsets.only(left: 10, right: 10),
              height: 28,
              width: 82,
              child: ElevatedButton(
                key: ValueKey("pathThroughButton"),
                style: ButtonStyle(
                    backgroundColor:
                        MaterialStateProperty.resolveWith<Color>((states) {
                      if (states.contains(MaterialState.disabled)) {
                        return Color(0x7f337eff);
                      }
                      return Color(0x7f337eff);
                    }),
                    padding: MaterialStateProperty.all(
                        EdgeInsets.symmetric(vertical: 0)),
                    shape: MaterialStateProperty.all(RoundedRectangleBorder(
                        side: BorderSide(color: Color(0x7f337eff)),
                        borderRadius: BorderRadius.all(Radius.circular(14))))),
                onPressed: sendPassThrough,
                child: Text(
                  "发送HTTP",
                  style: TextStyle(color: Colors.white, fontSize: 12),
                  textAlign: TextAlign.center,
                ),
              ),
            ),
          ]),
          Text(
            opResult,
            maxLines: 20,
            style: TextStyle(
                fontSize: 16.0,
                fontWeight: FontWeight.bold,
                color: Colors.black),
          ),
        ],
      ),
      backgroundColor: Color(0xffe2e2e2),
    );
  }
}
