//
//  NIMFBaseModel.h
//  nim
//
//  Created by Netease on 2019/8/9.
//

#import <Foundation/Foundation.h>
#import "NSDictionary+NIMFJson.h"
#import <YYModel/YYModel.h>
#import <NIMSDK/NIMSDK.h>

NS_ASSUME_NONNULL_BEGIN

#pragma mark - 基础模型
@interface NIMFBaseModel : NSObject

@property (nonatomic, strong) NSDictionary *dictionary;

/**
 receipt_id
 */
@property (nonatomic, copy) NSString *receiptId;

/**
 modlue
 */
@property (nonatomic, copy) NSString *module;

/**
 Dictionary -> Model

 @param dictionary dictionary
 @return Model
*/
+ (instancetype)modelWithDictionary:(NSDictionary *)dictionary;

/**
 Init with Model
 
 @param modelName model name
 @return Instance
 */
- (instancetype)initWithModelName:(NSString *)modelName;

/**
 Model -> Dictionary
 
 @return Dictionary
 */
- (NSMutableDictionary *)modelDictionary;

@end

#pragma mark - Flutter 向 iOS 传递的模型
@protocol NIMFRequestProtocol <NSObject>

/**
 Flutter Request -> SDK Request
 
 @return SDK Request
 */
- (id)setupSdkRequest;

@end

@interface NIMFBaseRequest : NIMFBaseModel<NIMFRequestProtocol>

/**
 Param Check
 
 @return valid
 */
- (BOOL)isValid;

@end

#pragma mark - iOS 向 Flutter 传递的模型
@protocol NIMFResponseProtocol <NSObject>

/**
 SDK Response -> Update Flutter Response
 
 @param response SDK response
 */
- (void)setSdkResponse:(id)response;

@end


@interface NIMFBaseResponse : NIMFBaseModel<NIMFResponseProtocol>

/**
 code
*/
@property (nonatomic, assign) NSInteger code;



/**
 Init With Request

 @param request Flutter Request
 @return Instance
 */
- (instancetype)initWithRequestModel:(NIMFBaseRequest *)request;

@end


NS_ASSUME_NONNULL_END
