//
//  NIMFServiceImp.m
//  nim
//
//  Created by Netease on 2019/8/8.
//

#import "NIMFServiceImp.h"
#import "NIMFBaseService.h"
#import "NIMFModuleDefine.h"
#import "NIMFModuleDefine.h"
#import "NIMFConfigHandler.h"
@interface NIMFServiceImp ()

@property (nonatomic, strong) NSMutableDictionary <NSString *, id<NIMFHandlerProtocol>>*handlerDic;
@property (nonatomic, assign)   BOOL        isInitHandler;   // 是否已经初始化了handler

@end

@implementation NIMFServiceImp

#pragma mark - Public
+ (void)startWithMethodChannel:(FlutterMethodChannel *)methodChannel {
    [NIMFServiceImp shareService].methodChannel = methodChannel;
}

+ (void)handleMethodCall:(FlutterMethodCall*)call
                  result:(FlutterResult)result {
    [[NIMFServiceImp shareService] handleMethodCall:call
                                                 result:result];
}

+ (instancetype)shareService {
    static id instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [[NIMFServiceImp alloc] init];
    });
    return instance;
}

- (instancetype)init {
    if (self = [super init]) {
        _handlerDic = [NSMutableDictionary dictionary];
//        [self doInitHandlers];
    }
    return self;
}

- (NSDictionary <NSString *, NSString *>*)allSupportHandlers {
    
    return @{
              //   Key   : Model name (Defined by Dart)
              //   Value : Class name (Handler Class)
             
              //1.配置服务
//              NIMFConfigServiceModule   : @"NIMFConfigRequestHandler",
              
              //2.登录服务
              NIMFLoginServiceModule    : @"NIMFLoginHandler",
              
              //3.信令服务
              NIMFSignalServiceModule   : @"NIMFSignalHandler",
              
              //4.好友服务
              NIMFUserServiceModule   : @"NIMFUserHandler",
              
              //5.透传服务
              NIMFPassThroughServiceModule   : @"NIMFPassThroughHandler",
              
              //6.消息服务
              NIMFMessageServiceModule : @"NIMFMessageHandler",
              
              //7.资源服务
              NIMFResourceServiceModule : @"NIMFResourceHandler",
              
              //8.聊天室
              NIMFChatroomServiceModule : @"NIMFChatroomHandler",
            };
}

#pragma mark - Hander查找
- (void)doInitHandlers {
    if(!_isInitHandler){
        
    NSDictionary *allHanders = [self allSupportHandlers];
    [allHanders enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
        Class cls = NSClassFromString(obj);
        id<NIMFHandlerProtocol> handler = [[cls alloc] init];
        if (handler) {
            _handlerDic[key] = handler;
        }
    }];
        _isInitHandler = YES;
    }
}

- (id<NIMFHandlerProtocol>)handlerWithModelName:(NSString *)modelName {
    id<NIMFHandlerProtocol> handler = _handlerDic[modelName];
    if (handler == nil) {
        NSLog(@"Module: [%@] not found", modelName);
    }
    return handler;
}

#pragma mark - 消息转发
- (void)handleMethodCall:(FlutterMethodCall*)call
                  result:(FlutterResult)result {
    id modelName = nil;
    NSDictionary *arguments;
    //获取模块id
    if ([call.method isEqualToString:@"config"]) {
//        modelName = NIMFConfigServiceModule;
        [[NIMFConfigRequestHandler sharedConfig] handleMethodCall:call result:result];
        //使用时进行初始化
        [self doInitHandlers];
    } else {
    arguments = call.arguments;
    modelName = arguments[@"module"];
    //模块校验
    if (!modelName || ![modelName isKindOfClass:[NSString class]]) {
        result(@(-1001));
        return;
    }
    //转发
    NSLog(@"[iOS] Call>>>>: Model:%@ Menthod:%@ ", modelName, call.method);
    id<NIMFHandlerProtocol> handler = [self handlerWithModelName:modelName];
    if (handler) {
        [handler handleMethodCall:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
    }
}

@end
