//
//  NIMFChatroomHandler.m
//  nim
//
//  Created by 张根宁 on 2020/6/17.
//

#import "NIMFChatroomHandler.h"
#import "NIMFChatroomObserver.h"
#import "NIMFChatroomAPI.h"
#import "NIMFEnterChatroomRequest.h"
#import "NIMFEnterChatroomResponse.h"
#import "NIMFCommonChatroomRequest.h"
#import "NIMFChatroomSendMsgRequest.h"
#import "NIMFFetchRoomInfoResponse.h"
#import "NIMFFetchChatroomMembersRequest.h"
#import "NIMFChatroomMessage.h"
#import "NIMFChatroomInfo.h"
#import "NIMFChatroomMember.h"
#import "NIMFCustomAttachmentDecoder.h"
#import "NIMFFetchRoomMembersResponse.h"
#import "NIMFUpdateMyChatroomMemberInfoRequest.h"


@interface NIMFChatroomHandler () <NIMChatManagerDelegate>

@property (nonatomic,strong) NSMutableDictionary<NSString *, NIMFBaseRequest *> * requests;

@end

@implementation NIMFChatroomHandler

- (void)dealloc
{
    [[NIMSDK sharedSDK].chatManager removeDelegate:self];
}

- (instancetype)init {
    if (self = [super init]) {
        self.observer = [[NIMFChatroomObserver alloc] init];
        [[NIMSDK sharedSDK].chatManager addDelegate:self];
        //[NIMCustomObject registerCustomDecoder:[[NIMFCustomAttachmentDecoder alloc] init]];
        _requests = [NSMutableDictionary dictionary];
    }
    return self;
}

- (id)NIMF_LOCAL_METHOD(enterChatRoom):(NSDictionary *)arguments {
    NIMFEnterChatroomRequest *request = [NIMFEnterChatroomRequest modelWithDictionary:arguments];
    NIMChatroomEnterRequest *sdkReq = [request sdkRequest];
    if (sdkReq.mode) {
        [NIMChatroomIndependentMode registerRequestChatroomAddressesHandler:^(NSString * _Nonnull roomId, NIMRequestChatroomAddressesCallback  _Nonnull callback) {
            if (callback) {
                callback(nil, request.links);
            }
        }];
    }
    
    [[NIMSDK sharedSDK].chatroomManager enterChatroom:sdkReq completion:^(NSError * _Nullable error, NIMChatroom * _Nullable chatroom, NIMChatroomMember * _Nullable me) {
        NIMFEnterChatroomResponse *response = [[NIMFEnterChatroomResponse alloc] initWithRequestModel:request];
        response.code = error ? error.code : 200;
        NIMFEnterChatroomResultData *data = [[NIMFEnterChatroomResultData alloc] init];
        data.roomId = sdkReq.roomId;
        data.resCode = error ? error.code : 200;
        if (!error) {
            data.chatRoomInfo = [[NIMFChatroomInfo alloc] initWithSdkChatroom:chatroom];
            data.chatRoomMember = [[NIMFChatroomMember alloc] initWithSdkChatroomMember:me];
            data.chatRoomMember.roomId = sdkReq.roomId;
            data.statusCode = NIMFLoginStateLogined;
        } else {
            data.statusCode = NIMFLoginStateUnlogin;
        }
        response.enterDataResult = data;
        [NIMFChatroomAPI onEnterChatroomCallback:response];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(exitChatRoom):(NSDictionary *)arguments {
    NIMFCommonChatroomRequest *request = [NIMFCommonChatroomRequest modelWithDictionary:arguments];
    
    [[NIMSDK sharedSDK].chatroomManager exitChatroom:request.roomId
                                          completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        response.code = error ? error.code : 200;
        [NIMFChatroomAPI onExitChatroomCallback:response];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(getEnterErrorCode):(NSDictionary *)arguments {
    return FlutterMethodNotImplemented;
}

- (id)NIMF_LOCAL_METHOD(sendMessage):(NSDictionary *)arguments {
    NIMFChatroomSendMsgRequest *request = [NIMFChatroomSendMsgRequest modelWithDictionary:arguments];
    NIMMessage *message = [request.message yx_message];
    NIMSession *session = [request.message yx_chatroomSession];
    if (message.messageId.length > 0)
    {
        self.requests[message.messageId] = request;
    }
    
    NSError * error = nil;
    if (!request.resend)
    {
        [[NIMSDK sharedSDK].chatManager sendMessage:message
                                          toSession:session
                                              error:&error];
    }
    else
    {
        [[NIMSDK sharedSDK].chatManager resendMessage:message
                                                error:&error];
    }

    return nil;
}

- (id)NIMF_LOCAL_METHOD(downloadAttachment):(NSDictionary *)arguments {
    return FlutterMethodNotImplemented;
}

- (id)NIMF_LOCAL_METHOD(pullMessageHistory):(NSDictionary *)arguments {
    return FlutterMethodNotImplemented;
}

- (id)NIMF_LOCAL_METHOD(fetchRoomInfo):(NSDictionary *)arguments {
    NIMFCommonChatroomRequest *request = [NIMFCommonChatroomRequest modelWithDictionary:arguments];
    [[NIMSDK sharedSDK].chatroomManager fetchChatroomInfo:request.roomId
                                               completion:^(NSError * _Nullable error, NIMChatroom * _Nullable chatroom) {
        NIMFFetchRoomInfoResponse *response = [[NIMFFetchRoomInfoResponse alloc] initWithRequestModel:request];
        if (error) {
            response.code = error.code;
        } else {
            response.roomInfo = [[NIMFChatroomInfo alloc] initWithSdkChatroom:chatroom];
        }
        [NIMFChatroomAPI onFetchRoomInfoCallback:response];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(fetchRoomMembers):(NSDictionary *)arguments {
    NIMFFetchChatroomMembersRequest *request = [NIMFFetchChatroomMembersRequest modelWithDictionary:arguments];
    NIMChatroomMemberRequest *yx_request = [request yx_request];
    
    [[NIMSDK sharedSDK].chatroomManager fetchChatroomMembers:yx_request
                                                  completion:^(NSError * _Nullable error, NSArray<NIMChatroomMember *> * _Nullable members) {
        NIMFFetchRoomMembersResponse *response = [[NIMFFetchRoomMembersResponse alloc] initWithRequestModel:request];
        if (error) {
            response.code = error.code;
        } else {
            NSMutableArray *ret = [NSMutableArray array];
            [members enumerateObjectsUsingBlock:^(NIMChatroomMember * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                NIMFChatroomMember *meb = [[NIMFChatroomMember alloc] initWithSdkChatroomMember:obj];
                [ret addObject:meb];
            }];
            response.members = ret;
        }
        [NIMFChatroomAPI onFetchRoomMembersCallback:response];
    }];
    
    return nil;
}

- (id)NIMF_LOCAL_METHOD(updateMyRoomRole):(NSDictionary *)arguments {
    NIMFUpdateMyChatroomMemberInfoRequest *request = [NIMFUpdateMyChatroomMemberInfoRequest modelWithDictionary:arguments];
    NIMChatroomMemberInfoUpdateRequest *yx_request = [request yx_request];
    [[NIMSDK sharedSDK].chatroomManager updateMyChatroomMemberInfo:yx_request
                                                  completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        response.code = error ? error.code : 200;
        [NIMFChatroomAPI onUpdateChatroomMemberInfoCallback:response];
    }];
    
    return nil;
}

#pragma mark - NIMChatManagerDelegate
- (void)sendMessage:(NIMMessage *)message
           progress:(float)progress
{
    
}

- (void)sendMessage:(NIMMessage *)message
    didCompleteWithError:(nullable NSError *)error
{
    NIMFCommonChatroomRequest * request = (NIMFCommonChatroomRequest *)self.requests[message.messageId];
    if (!request)
    {
        return;
    }
    self.requests[message.messageId] = nil;
    NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
    if (error)
    {
        response.code = error.code;
    }
    [NIMFChatroomAPI onSendMessageCallback:response];
}

@end
