//
//  NIMFFriendHandler.m
//  nim
//
//  Created by He on 2019/12/10.
//

#import "NIMFUserHandler.h"
#import "NIMFUserRequest.h"
#import "NIMFUserFlutterAPI.h"
#import "NIMFUserObserver.h"
#import "NIMFUser.h"
#import "NIMFFriendResponse.h"

@implementation NIMFUserHandler

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        self.observer = [[NIMFUserObserver alloc] init];
    }
    return self;
}

/**
 *  请求添加好友
 */
- (id)NIMF_LOCAL_METHOD(requestFriend):(NSDictionary *)arguments
{
    NIMFFriendRequest * request = [NIMFFriendRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    NIMUserRequest * userRequest = [request setupSdkRequest];
    [[NIMSDK sharedSDK].userManager requestFriend:userRequest
                                       completion:^(NSError * error)
    {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFFriendFlutterAPI sendMethod:@"onRequestFriend"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}

/**
 *  删除好友
 */
- (id)NIMF_LOCAL_METHOD(deleteFriend):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].userManager deleteFriend:request.userId
                                     removeAlias:request.removeAlias
                                      completion:^(NSError * error)
    {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFFriendFlutterAPI sendMethod:@"onDeleteFriend"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}

/**
 *  返回我的好友列表
 */
- (id)NIMF_LOCAL_METHOD(myFriends):(NSDictionary *)arguments
{
    NSArray * array = [[NIMSDK sharedSDK].userManager myFriends];
    return [array yy_modelToJSONObject];
}

/**
 *  是否是我的好友
 */
- (id)NIMF_LOCAL_METHOD(isMyFriend):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    BOOL isFriend = [[NIMSDK sharedSDK].userManager isMyFriend:request.userId];
    return @(isFriend);
}

/**
 *  添加用户到黑名单
 */
- (id)NIMF_LOCAL_METHOD(addToBlackList):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].userManager addToBlackList:request.userId
                                        completion:^(NSError * error)
    {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFFriendFlutterAPI sendMethod:@"onAddToBlackList"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}

/**
 *  将用户从黑名单移除
 */
- (id)NIMF_LOCAL_METHOD(removeFromBlackBlackList):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].userManager addToBlackList:request.userId
                                        completion:^(NSError * error)
    {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFFriendFlutterAPI sendMethod:@"onRemoveFromBlackList"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}

/**
 *  判断用户是否已被拉黑
 */
- (id)NIMF_LOCAL_METHOD(isUserInBlackList):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    BOOL InBlackList = [[NIMSDK sharedSDK].userManager isUserInBlackList:request.userId];
    return @(InBlackList);
}

/**
 *  返回我的黑名单列表
 */
- (id)NIMF_LOCAL_METHOD(myBlackList):(NSDictionary *)arguments
{
    NSArray * array = [[NIMSDK sharedSDK].userManager myBlackList];
    return [array yy_modelToJSONObject];
}

/**
 *  设置消息提醒
 */
- (id)NIMF_LOCAL_METHOD(updateNotifyState):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].userManager updateNotifyState:request.notify
                                              forUser:request.userId
                                           completion:^(NSError * error)
    {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFFriendFlutterAPI sendMethod:@"onUpdateNotifyState"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}

/**
 *  是否需要消息提醒
 */
- (id)NIMF_LOCAL_METHOD(notifyForNewMsg):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    BOOL notify = [[NIMSDK sharedSDK].userManager notifyForNewMsg:request.userId];
    return @(notify);
}

/**
 *  禁言列表
 */
- (id)NIMF_LOCAL_METHOD(myMuteUserList):(NSDictionary *)arguments
{
    NSArray * array = [[NIMSDK sharedSDK].userManager myMuteUserList];
    return [array yy_modelToJSONObject];
}

/**
 *  云端批量获取好友信息
 */
- (id)NIMF_LOCAL_METHOD(fetchUsers):(NSDictionary *)arguments
{
    NIMFFetchFriendsRequest * request = [NIMFFetchFriendsRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].userManager fetchUserInfos:request.userIds
                                        completion:^(NSArray<NIMUser *> * _Nullable users, NSError * _Nullable error) {
        NIMFFetchFriendsResponse * response = [[NIMFFetchFriendsResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [response setSdkResponse:users];
        [NIMFFriendFlutterAPI sendMethod:@"onFetchUsers"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}

/**
 * 从本地获取用户资料
 */
- (id)NIMF_LOCAL_METHOD(friendById):(NSDictionary *)arguments
{
    NIMFBaseUserRequest * request = [NIMFBaseUserRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    return [[[NIMSDK sharedSDK].userManager userInfo:request.userId] yy_modelToJSONObject];
}


/**
 *  更新好友信息，主要是备注名之类的
 */
- (id)NIMF_LOCAL_METHOD(updateFriend):(NSDictionary *)arguments
{
    NIMFFriendUpdateRequest * request = [NIMFFriendUpdateRequest modelWithDictionary:arguments];
    if (!request.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].userManager updateUser:request.target.nimUser
                                    completion:^(NSError * _Nullable error) {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFFriendFlutterAPI sendMethod:@"onUpdateFriend"
                               arguments:[response modelDictionary]];
    }];
    return nil;
}


@end
