//
//  NIMFLoginHandler.m
//  nim
//
//  Created by Netease on 2019/8/8.
//

#import "NIMFLoginHandler.h"
#import "NIMFAuthRequest.h"
#import "NIMFLoginObserver.h"
#import "NIMFLoginFlutterAPI.h"
#import "NIMFLoginUntil.h"
#import "NIMFLoginResponse.h"
#import "NIMFKickOtherRequst.h"

@implementation NIMFLoginHandler

- (instancetype)init {
    if (self = [super init]) {
        self.observer = [[NIMFLoginObserver alloc] init];
    }
    return self;
}

- (id)NIMF_LOCAL_METHOD(login):(NSDictionary *)arguments {
    NIMFAuthRequest *authInfo = [NIMFAuthRequest modelWithDictionary:arguments];
    if (!authInfo.isValid) {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].loginManager login:authInfo.account
                                     token:authInfo.token
                                completion:^(NSError * _Nullable error)
    {
        if (error) {
            NIMFLoginResponse *ret = [[NIMFLoginResponse alloc] initWithRequestModel:authInfo];
            ret.statusCode = [NIMFLoginUntil stateWithLoginErrorCode:error.code];
            [NIMFLoginFlutterAPI onOnlineStatus:ret];
        }
        
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:authInfo];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFLoginFlutterAPI onLogin:response];
        NSLog(@"[iOS] 登入完成。error:%@", error);
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(logout):(NSDictionary *)arguments {
    NIMFBaseRequest *request = [NIMFBaseRequest modelWithDictionary:arguments];
    [[NIMSDK sharedSDK].loginManager logout:^(NSError * _Nullable error) {
        NIMFLoginResponse *req = [[NIMFLoginResponse alloc] initWithRequestModel:request];
        if (error) {
            req.statusCode = error.code;
        }
        [NIMFLoginFlutterAPI onDefaultResponse:req];
        NSLog(@"[iOS] 登出完成。error:%@", error);
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(kickOtherClient):(NSDictionary *)arguments {
    NIMFKickOtherRequst *request = [NIMFKickOtherRequst modelWithDictionary:arguments];
    
    NIMLoginClient *client = [request setupSdkRequest];
    [[NIMSDK sharedSDK].loginManager kickOtherClient:client
                                          completion:^(NSError * _Nullable error)
    {
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFLoginFlutterAPI onKickOtherClient:response];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(getCurrentAccount):(NSDictionary *)arguments {
    return [[NIMSDK sharedSDK].loginManager currentAccount];
}

- (id)NIMF_LOCAL_METHOD(getLoginInfo):(NSDictionary *)arguments {
    NSMutableDictionary *ret = [NSMutableDictionary dictionary];
    NSString *currentAccount = [[NIMSDK sharedSDK].loginManager currentAccount];
    ret[@"account"] = currentAccount;
    return ret;
}

@end
