//
//  NIMFLoginObserver.m
//  nim
//
//  Created by Netease on 2019/8/12.
//

#import "NIMFLoginObserver.h"
#import "NIMFLoginUntil.h"
#import "NIMFLoginFlutterAPI.h"
#import "NIMFLoginResponse.h"

@interface NIMFLoginObserver ()<NIMLoginManagerDelegate>
@end

@implementation NIMFLoginObserver

- (void)dealloc {
    [[NIMSDK sharedSDK].loginManager removeDelegate:self];
}

- (instancetype)init {
    if (self = [super init]) {
        [[NIMSDK sharedSDK].loginManager addDelegate:self];
    }
    return self;
}

//被踢(服务器/其他端)回调
- (void)onKick:(NIMKickReason)code clientType:(NIMLoginClientType)clientType {
    NIMFLoginResponse *res = [[NIMFLoginResponse alloc] init];
    switch (code) {
        case NIMKickReasonByClient:
            res.statusCode = NIMFLoginStateKickByClientManually;
            break;
        case NIMKickReasonByClientManually:
            res.statusCode = NIMFLoginStateKickByClientManually;
            break;
        case  NIMKickReasonByServer:
           res.statusCode = NIMFLoginStateKickByClient;
        default:
            break;
    }
    [NIMFLoginFlutterAPI onOnlineStatus:res];
}

//登录回调
- (void)onLogin:(NIMLoginStep)step {
    NIMFLoginResponse *res = [[NIMFLoginResponse alloc] init];
    switch (step) {
        case NIMLoginStepLinking:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepLinking");
            res.statusCode = NIMFLoginStateConnecting;
            [NIMFLoginFlutterAPI onOnlineStatus:res];
            break;
        }
        case NIMLoginStepLinkOK:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepLinkOK");
            res.statusCode = NIMFLoginStateLogining;
            [NIMFLoginFlutterAPI onOnlineStatus:res];
            break;
        }
        case NIMLoginStepLinkFailed:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepLinkFailed");
            res.statusCode = NIMFLoginStateUnlogin;
            [NIMFLoginFlutterAPI onOnlineStatus:res];
            break;
        }
        case NIMLoginStepLogining:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepLogining");
            res.statusCode = NIMFLoginStateLogining;
            [NIMFLoginFlutterAPI onOnlineStatus:res];
            break;
        }
        case NIMLoginStepLoginOK:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepLoginOK");
            res.statusCode = NIMFLoginStateLogined;
            [NIMFLoginFlutterAPI onOnlineStatus:res];
            break;
        }
        case NIMLoginStepLoginFailed:
        {
            //回调返回
            NSLog(@"[iOS] onLogin:NIMLoginStepLoginFailed");
            break;
        }
        case NIMLoginStepSyncing:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepSyncing");
            break;
        }
        case NIMLoginStepSyncOK:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepSyncOK");
            break;
        }
        case NIMLoginStepLoseConnection:
        {
            NSLog(@"[iOS] onLogin:NIMLoginStepLoseConnection");
            res.statusCode = NIMFLoginStateNetBroken;
            [NIMFLoginFlutterAPI onOnlineStatus:res];
            break;
        }
        case NIMLoginStepNetChanged:
        default:
            break;
    }
}

//登录失败回调
- (void)onAutoLoginFailed:(NSError *)error {
    NIMFLoginResponse *ret = [[NIMFLoginResponse alloc] init];
    ret.statusCode = [NIMFLoginUntil stateWithLoginErrorCode:error.code];
    [NIMFLoginFlutterAPI onOnlineStatus:ret];
}

//多端登录发生变化
- (void)onMultiLoginClientsChanged {
    
}

//群用户同步完成通知
- (void)onTeamUsersSyncFinished:(BOOL)success {
    
}

//超大群用户同步完成通知
- (void)onSuperTeamUsersSyncFinished:(BOOL)success {
    
}

@end
