//
//  NIMFMessage.m
//  nim
//
//  Created by He on 2019/12/25.
//

#import "NIMFMessage.h"
#import "NIMFStringAttachment.h"
#import "NIMFImageAttachment.h"

@implementation NIMFMessage

- (instancetype)initWithYXMessage:(NIMMessage *)message
{
    self = [super init];
    if (self)
    {
        [self setupWithMessage:message];
    }
    return self;
}

- (void)setupWithMessage:(NIMMessage *)message
{
    self.uuid = message.messageId;
    self.sessionId = message.session.sessionId;
    self.sessionType =  [self f_sessionTypeWithMessage:message];
    self.msgStatus = [self f_msgStaus:message];
    self.msgType = [self f_msgType:message];
    self.msgDirection = [self f_msgDirecetion:message];
    self.content = message.text;
    self.time = (int64_t)message.timestamp * 1000;
    self.fromAccount = message.from;
    self.attachStatus = [self f_attachStatus:message];
    self.remoteExt = [message.remoteExt yy_modelToJSONString];
    self.localExt = [message.localExt yy_modelToJSONString];
    self.pushContent = message.apnsContent;
    self.pushPayload = [message.apnsPayload yy_modelToJSONString];
    self.serverId = [message.serverID integerValue];
    self.hasSendAck = message.isTeamReceiptSended;
    switch (message.messageType) {
//        case NIMMessageTypeCustom:
//        {
//            NIMCustomObject *customObject = message.messageObject;
//            NIMFStringAttachment *attach = customObject.attachment;
//            self.attach = attach.content;
//        }
//            break;
        case NIMMessageTypeImage:
        {
            NIMImageObject *fileObject = message.messageObject;
            NIMFImageAttachment *file = [[NIMFImageAttachment alloc] init];
            file.name = fileObject.displayName;
            file.url = fileObject.url;
            file.path = fileObject.path;
            file.size = fileObject.fileLength;
            file.md5 = fileObject.md5;
            file.width = fileObject.size.width;
            file.height = fileObject.size.height;
            self.attach = [file yy_modelToJSONString];
        }
            break;
        case NIMMessageTypeFile:
        {
            NIMFileObject *fileObject = message.messageObject;
            NIMFFileAttachment *file = [[NIMFFileAttachment alloc] init];
            file.name = fileObject.displayName;
            file.url = fileObject.url;
            file.path = fileObject.path;
            file.size = fileObject.fileLength;
            file.md5 = fileObject.md5;
            self.attach = [file yy_modelToJSONString];
        }
            break;
            
            
        default:
            break;
    }
}

// 只有部分可写属性
- (NIMMessage *)yx_message
{
    NIMMessage * message = [[NIMMessage alloc] init];
//    message.messageId = self.uuid;
//    message.session
//    message.messageType
//    message.deliveryState
//    message.isOutgoingMsg = self.msgDirection == 0 ? true : false;
//    message.isReceivedMsg = !message.isOutgoingMsg;
//    message.messageObject
//    message.serverID
//    message.attachmentDownloadState = self.attachStatus;
    
    
    if (self.remoteExt.length > 0 &&
        [self.remoteExt isKindOfClass:[NSString class]])
    {
        NSData * data = [self.remoteExt dataUsingEncoding:NSUTF8StringEncoding];
        if (data)
        {
            message.remoteExt = [NSJSONSerialization JSONObjectWithData:data
                                                                options:NSJSONReadingFragmentsAllowed
                                                                  error:nil];
        }
        
    }

    if (self.localExt.length > 0 &&
        [self.localExt isKindOfClass:[NSString class]])
    {
        NSData * data = [self.localExt dataUsingEncoding:NSUTF8StringEncoding];
        if (data)
        {
            message.localExt = [NSJSONSerialization JSONObjectWithData:data
                                                               options:NSJSONReadingFragmentsAllowed
                                                                 error:nil];
        }
    }
    
    message.text = self.content;
    message.timestamp = self.time/1000.0;
    message.from = self.fromAccount;
    message.apnsContent = self.pushContent;
    
    if (self.pushPayload.length > 0)
    {
        NSData * data = [self.pushPayload dataUsingEncoding:NSUTF8StringEncoding];
        if (data)
        {
            message.apnsPayload = [NSJSONSerialization JSONObjectWithData:data
                                                                  options:NSJSONReadingFragmentsAllowed
                                                                    error:nil];
        }
    }
    switch(self.msgType)
    {
//        case NIMFMsgTypeCustom:
//        {
//            NIMFStringAttachment *attachment = [[NIMFStringAttachment alloc] init];
//            attachment.content = self.attach;
//            NIMCustomObject *customObject  = [[NIMCustomObject alloc] init];
//            customObject.attachment = attachment;
//            message.messageObject = customObject;
//        }
//            break;
        case NIMFMsgTypeImage:
        {
            NIMFImageAttachment *flutterImageObject = [NIMFImageAttachment yy_modelWithJSON:self.attach];
            
            NSString *scene = [self sceneWithFlutterScene:flutterImageObject.nosTokenSceneKey];
            NIMImageObject *imageObject = [[NIMImageObject alloc] initWithFilepath:flutterImageObject.path scene:scene];
            imageObject.displayName = flutterImageObject.name;
            
            NIMImageOption *option  = [[NIMImageOption alloc] init];
            option.compressQuality  = 0.8;
            imageObject.option = option;
            message.messageObject   = imageObject;
            
            NIMMessageSetting *setting = [[NIMMessageSetting alloc] init];
            setting.scene = scene;
            message.setting = setting;
        }
            break;
        case NIMFMsgTypeFile:
        {
            NIMFFileAttachment *flutterFileObject = [NIMFFileAttachment yy_modelWithJSON:self.attach];
            NSString *scene = [self sceneWithFlutterScene:flutterFileObject.nosTokenSceneKey];
            
            NIMFileObject *fileObject = [[NIMFileObject alloc] initWithSourcePath:flutterFileObject.path scene:scene];
            fileObject.displayName    = flutterFileObject.name;
            message.messageObject     = fileObject;
            
            NIMMessageSetting *setting = [[NIMMessageSetting alloc] init];
            setting.scene = scene;
            message.setting = setting;
             
        }
            break;
        default:
            break;
    }
    return message;
}

#pragma mark - YunXin SDK Property

- (NIMSession *)yx_session
{
    NIMSession * session = [NIMSession session:self.sessionId
                                          type:[self yx_sessionType]];
    return session;
}

- (NIMSession *)yx_chatroomSession
{
    NIMSession * session = [NIMSession session:self.sessionId
                                          type:NIMSessionTypeChatroom];
    return session;
}

// 值未完全对齐
- (NIMSessionType)yx_sessionType
{
    if (self.sessionType == 0)
    {
        return NIMSessionTypeP2P;
    }
    else if (self.sessionType == 1)
    {
        return NIMSessionTypeTeam;
    }
    else if (self.sessionType == 5)
    {
        return NIMSessionTypeSuperTeam;
    }
    else if (self.sessionType == 10002)
    {
        return NIMSessionTypeChatroom;
    }
    
    return self.sessionType;
}

#pragma mark - Flutter SDK Property

- (int)f_sessionTypeWithMessage:(NIMMessage *)message
{
    NIMSessionType type = message.session.sessionType;
    if (type == NIMSessionTypeP2P)
    {
        return 0;
    }
    else if (type == NIMSessionTypeTeam)
    {
        return 1;
    }
    else if (type == NIMSessionTypeSuperTeam)
    {
        return 5;
    }
    else if (type == NIMSessionTypeChatroom)
    {
        return 10002;
    }
    
    return -1;
}

// 值未完全对齐
- (int)f_msgStaus:(NIMMessage *)message
{
    int status = -1;

    if (message.deliveryState == NIMMessageDeliveryStateDelivering)
    {
        status = 0;
    }
    
    if (message.deliveryState == NIMMessageDeliveryStateDeliveried)
    {
        status = 1;
    }
    
    if (message.deliveryState == NIMMessageDeliveryStateFailed)
    {
        status = 2;
    }
    
    
    return status;
}

- (int)f_msgDirecetion:(NIMMessage *)message
{
    int direction = 0;
    if (!message.isOutgoingMsg)
    {
        direction = 1;
    }
    return direction;
}

// 值未完全对齐
- (int)f_attachStatus:(NIMMessage *)message
{
    int status = 0;
    switch (message.attachmentDownloadState) {
        case NIMMessageAttachmentDownloadStateNeedDownload:
            status = 0;
            break;
        case NIMMessageAttachmentDownloadStateDownloading:
            status = 1;
            break;
        case NIMMessageAttachmentDownloadStateDownloaded:
            status = 2;
            break;
        case NIMMessageAttachmentDownloadStateFailed:
            status = 3;
            break;
        default:
            status = 4; // cancel这个状态iOS端没有
            break;
    }
    return status;
}

// 值未完全对齐
- (NSInteger)f_msgType:(NIMMessage *)message
{
    int type = NIMFMsgTypeUndef;
    switch (message.messageType) {
        case NIMMessageTypeText:
            type = NIMFMsgTypeText;
            break;
        case NIMMessageTypeImage:
            type = NIMFMsgTypeImage;
            break;
        case NIMMessageTypeAudio:
            type = NIMFMsgTypeAudio;
            break;
        case NIMMessageTypeVideo:
            type = NIMFMsgTypeVideo;
            break;
        case NIMMessageTypeLocation:
            type = NIMFMsgTypeLocation;
            break;
        case NIMMessageTypeNotification:
            type = NIMFMsgTypeNotification;
            break;
        case NIMMessageTypeFile:
            type = NIMFMsgTypeFile;
            break;
//        case NIMMessageType:
//            type = 7;
//            break;
//        case NIMMessageTypeText:
//            type = 8;
//            break;
        case NIMMessageTypeTip:
            type = NIMFMsgTypeTip;
            break;
        case NIMMessageTypeRobot:
            type = NIMFMsgTypeRobot;
            break;
        case NIMMessageTypeCustom:
            type = NIMFMsgTypeCustom;
            break;
        default:
            break;
    }
    return type;
}

#pragma mark - private
- (NSString *)sceneWithFlutterScene:(NSString *)scene {
    NSString *imScene = NIMNOSSceneTypeMessage;
    if ([scene isEqual:@"nim_icon"]) {
        imScene = NIMNOSSceneTypeAvatar;
    }else if([scene isEqual:@"nim_security"]) {
        imScene = NIMNOSSceneTypeSecurity;
    }
    return imScene;
}
@end
