//
//  NIMFMessageHandler.m
//  nim
//
//  Created by He on 2019/12/25.
//

#import "NIMFMessageHandler.h"
#import "NIMFMessageObserver.h"
#import "NIMFMessageRequest.h"
#import "NIMFMessage.h"
#import "NIMFMessageResponse.h"
#import "NIMFMessageFlutterAPI.h"
#import "NIMFCustomAttachmentDecoder.h"

@interface NIMFMessageHandler () <NIMChatManagerDelegate>

@property (nonatomic,strong) NSMutableDictionary<NSString *, NIMFBaseRequest *> * requests;

@end

@implementation NIMFMessageHandler

- (void)dealloc
{
    [[NIMSDK sharedSDK].chatManager removeDelegate:self];
}

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        self.observer = [[NIMFMessageObserver alloc] init];
        [[NIMSDK sharedSDK].chatManager addDelegate:self];
        //[NIMCustomObject registerCustomDecoder:[[NIMFCustomAttachmentDecoder alloc] init]];
        _requests = [NSMutableDictionary dictionary];
    }
    return self;
}

- (id)NIMF_LOCAL_METHOD(sendMessage):(NSDictionary *)arguments
{
    NIMFMessageRequest * request = [NIMFMessageRequest modelWithDictionary:arguments];
    NIMMessage * message = request.message.yx_message;
    NIMSession * session = request.message.yx_session;
    
    dispatch_block_t mainBlock = ^ {
        if (message.messageId.length > 0)
        {
            self.requests[message.messageId] = request;
        }
        
        NSError * error = nil;
        if (!request.resend)
        {
            [[NIMSDK sharedSDK].chatManager sendMessage:message
                                              toSession:session
                                                  error:&error];
        }
        else
        {
            [[NIMSDK sharedSDK].chatManager resendMessage:message
                                                    error:&error];
        }
    };
    nimf_main_async_safe(mainBlock);
    return nil;
}


#pragma mark - NIMChatManagerDelegate

- (void)sendMessage:(NIMMessage *)message
           progress:(float)progress
{
    
}

- (void)sendMessage:(NIMMessage *)message
    didCompleteWithError:(nullable NSError *)error
{
    dispatch_block_t mainBlock = ^ {
        NIMFMessageRequest * request = (NIMFMessageRequest *)self.requests[message.messageId];
        
        if (!request)
        {
            return;
        }
        self.requests[message.messageId] = nil;
        NIMFBaseResponse * response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        [NIMFMessageFlutterAPI onSendMessage:response];
    };
    nimf_main_async_safe(mainBlock);
}

@end
