//
//  NIMFResourceHandler.m
//  nim
//
//  Created by He on 2020/1/3.
//

#import "NIMFResourceHandler.h"
#import "NIMFResourceUploadRequest.h"
#import "NIMFResourceUploadResponse.h"
#import "NIMFResourceFlutterAPI.h"
#import "NIMFProgressResponse.h"
#import "NIMFResourceDownloadRequest.h"
#import "NIMFResourceRequest.h"
#import "NIMFOriginUrlRequest.h"
#import "NIMFResourceResponse.h"
#import "NIMFResourceObserver.h"

@implementation NIMFResourceHandler

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        self.observer = [[NIMFResourceObserver alloc] init];
    }
    return self;
}

- (id)NIMF_LOCAL_METHOD(uploadAtScene):(NSDictionary *)arguments
{
    NIMFResourceUploadRequest *request = [NIMFResourceUploadRequest modelWithDictionary:arguments];
    if (!request.isValid)
    {
        return @(NIMFInputParamError);
    }
    
    NIMResourceExtraInfo *extraInfo = [[NIMResourceExtraInfo alloc] init];
    extraInfo.md5 = request.md5;
    extraInfo.roomId = request.roomId;
    extraInfo.scene = request.sceneKey;
    
    if (extraInfo.scene.length == 0)
    {
        extraInfo.scene = NIMNOSSceneTypeMessage;
    }
    [[NIMSDK sharedSDK].resourceManager upload:request.filePath
                                     extraInfo:extraInfo
                                      progress:^(float progress)
     {
        NIMFProgressResponse *progressResponse = [[NIMFProgressResponse alloc] init];
        progressResponse.key = request.filePath;
        [progressResponse setSdkResponse:@(progress)];
        [NIMFResourceFlutterAPI onUploadProgress:progressResponse];
     }
                                    completion:^(NSString * urlString, NSError * error)
     {
        NIMFResourceUploadResponse *response = [[NIMFResourceUploadResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        
        [response setSdkResponse:urlString];
        [NIMFResourceFlutterAPI onUpload:response];
     }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(getOriginUrlFromShortUrl):(NSDictionary *)arguments
{
    NIMFOriginUrlRequest *request = [NIMFOriginUrlRequest modelWithDictionary:arguments];
    if (!request.isValid)
    {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].resourceManager fetchNOSURLWithURL:request.shortUrl
                                                    roomId:request.roomId
                                                completion:^(NSError * _Nullable error, NSString * _Nullable urlString)
    {
        NIMFResourceResponse *response = [[NIMFResourceResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        
        [response setSdkResponse:urlString];
        [NIMFResourceFlutterAPI onGetOriginUrlFromShortUrl:response];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(convertDownloadUrlToCDNUrl):(NSDictionary *)arguments
{
    NIMFResourceRequest *request = [NIMFResourceRequest modelWithDictionary:arguments];
    if (!request.isValid)
    {
        return @(NIMFInputParamError);
    }
    
    NSString *url = [[NIMSDK sharedSDK].resourceManager convertURLToAcceleratedURL:request.url];
    NIMFResourceResponse *response = [[NIMFResourceResponse alloc] initWithRequestModel:request];
    [response setSdkResponse:url];
    [NIMFResourceFlutterAPI onConvertDownloadUrlToCDNUrl:response];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(download):(NSDictionary *)arguments
{
    NIMFResourceDownloadRequest *request = [NIMFResourceDownloadRequest modelWithDictionary:arguments];
    if (!request.isValid)
    {
        return @(NIMFInputParamError);
    }
    
    NIMResourceExtraInfo *extraInfo = [[NIMResourceExtraInfo alloc] init];
    extraInfo.md5 = request.md5;
    extraInfo.roomId = request.roomId;
    extraInfo.scene = request.scene;
    [[NIMSDK sharedSDK].resourceManager download:request.url
                                        filepath:request.path
                                       extraInfo:extraInfo
                                        progress:^(float progress)
    {
        NIMFProgressResponse *progressResponse = [[NIMFProgressResponse alloc] initWithRequestModel:request];
        progressResponse.key = request.url;
        [progressResponse setSdkResponse:@(progress)];
        [NIMFResourceFlutterAPI onDownloadProgress:progressResponse];
    }
                                      completion:^(NSError * _Nullable error)
    {
        NIMFBaseResponse *response = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error)
        {
            response.code = error.code;
        }
        
        [NIMFResourceFlutterAPI onDownload:response];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(cancelTask):(NSDictionary *)arguments
{
    NIMFResourceRequest *request = [NIMFResourceRequest modelWithDictionary:arguments];
    if (!request.isValid)
    {
        return @(NIMFInputParamError);
    }
    
    [[NIMSDK sharedSDK].resourceManager cancelTask:request.path];
    return @(YES);
}

@end
