//
//  NIMFSignalHandler.m
//  nim
//
//  Created by Netease on 2019/9/20.
//

#import "NIMFSignalHandler.h"
#import "NIMFSignalFlutterAPI.h"
#import "NIMFSignalCreateRequest.h"
#import "NIMFSignalCloseRequest.h"
#import "NIMFSignalJoinRequest.h"
#import "NIMFSignalLeaveRequest.h"
#import "NIMFSignalInviteRequest.h"
#import "NIMFSignalCancelInviteRequest.h"
#import "NIMFSignalRejectRequest.h"
#import "NIMFSignalAcceptRequest.h"
#import "NIMFSignalControlRequest.h"
#import "NIMFSignalCallRequest.h"
#import "NIMFSignalQueryChannelRequest.h"
#import "NIMFSignalObserver.h"

@implementation NIMFSignalHandler

- (instancetype)init
{
    if (self = [super init]) {
        self.observer = [[NIMFSignalObserver alloc] init];
    }
    return self;
}

- (id)NIMF_LOCAL_METHOD(create):(NSDictionary *)arguments {
    NIMFSignalCreateRequest *request = [NIMFSignalCreateRequest modelWithDictionary:arguments];
    NIMSignalingCreateChannelRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingCreateChannel:sdkReq completion:^(NSError * _Nullable error, NIMSignalingChannelInfo * _Nullable response) {
        NIMFChannelInfoResponse *ret = [[NIMFChannelInfoResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        if (response) {
            [ret setSdkResponse:response];
        }
        [NIMFSignalFlutterAPI onCreate:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(close):(NSDictionary *)arguments {
    NIMFSignalCloseRequest *request = [NIMFSignalCloseRequest modelWithDictionary:arguments];
    NIMSignalingCloseChannelRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingCloseChannel:sdkReq completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(join):(NSDictionary *)arguments {
    NIMFSignalJoinRequest *request = [NIMFSignalJoinRequest modelWithDictionary:arguments];
    NIMSignalingJoinChannelRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingJoinChannel:sdkReq completion:^(NSError * _Nullable error, NIMSignalingChannelDetailedInfo * _Nullable response) {
        NIMFChannelFullInfoResponse *ret = [[NIMFChannelFullInfoResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        if (response) {
            [ret setSdkResponse:response];
        }
        [NIMFSignalFlutterAPI onJoin:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(leave):(NSDictionary *)arguments {
    NIMFSignalLeaveRequest *request = [NIMFSignalLeaveRequest modelWithDictionary:arguments];
    NIMSignalingLeaveChannelRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingLeaveChannel:sdkReq completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    
    return nil;
}

- (id)NIMF_LOCAL_METHOD(invite):(NSDictionary *)arguments {
    NIMFSignalInviteRequest *request = [NIMFSignalInviteRequest modelWithDictionary:arguments];
    NIMSignalingInviteRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingInvite:sdkReq completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(cancelInvite):(NSDictionary *)arguments {
    NIMFSignalCancelInviteRequest *request = [NIMFSignalCancelInviteRequest modelWithDictionary:arguments];
    NIMSignalingCancelInviteRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingCancelInvite:sdkReq completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(rejectInvite):(NSDictionary *)arguments {
    NIMFSignalRejectRequest *request = [NIMFSignalRejectRequest modelWithDictionary:arguments];
    NIMSignalingRejectRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingReject:sdkReq completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(acceptInvite):(NSDictionary *)arguments {
    NIMFSignalAcceptRequest *request = [NIMFSignalAcceptRequest modelWithDictionary:arguments];
    NIMSignalingAcceptRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingAccept:sdkReq completion:^(NSError * _Nullable error, NIMSignalingChannelDetailedInfo * _Nullable response) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(acceptInviteAndJoin):(NSDictionary *)arguments {
    NIMFSignalAcceptAndJoinRequest *request = [NIMFSignalAcceptAndJoinRequest modelWithDictionary:arguments];
    NIMSignalingAcceptRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingAccept:sdkReq completion:^(NSError * _Nullable error, NIMSignalingChannelDetailedInfo * _Nullable response) {
        NIMFChannelFullInfoResponse *ret = [[NIMFChannelFullInfoResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        if (response) {
            [ret setSdkResponse:response];
        }
        [NIMFSignalFlutterAPI onAcceptInviteAndJoin:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(sendControl):(NSDictionary *)arguments {
    NIMFSignalControlRequest *request = [NIMFSignalControlRequest modelWithDictionary:arguments];
    NIMSignalingControlRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingControl:sdkReq completion:^(NSError * _Nullable error) {
        NIMFBaseResponse *ret = [[NIMFBaseResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        [NIMFSignalFlutterAPI onDefaultResponse:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(call):(NSDictionary *)arguments {
    NIMFSignalCallRequest *request = [NIMFSignalCallRequest modelWithDictionary:arguments];
    NIMSignalingCallRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingCall:sdkReq completion:^(NSError * _Nullable error, NIMSignalingChannelDetailedInfo * _Nullable response) {
        NIMFChannelFullInfoResponse *ret = [[NIMFChannelFullInfoResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        if (response) {
            [ret setSdkResponse:response];
        }
        [NIMFSignalFlutterAPI onCall:ret];
    }];
    return nil;
}

- (id)NIMF_LOCAL_METHOD(queryChannelInfo):(NSDictionary *)arguments {
    NIMFSignalQueryChannelRequest *request = [NIMFSignalQueryChannelRequest modelWithDictionary:arguments];
    NIMSignalingQueryChannelRequest *sdkReq = [request setupSdkRequest];
    [self.signalManager signalingQueryChannelInfo:sdkReq completion:^(NSError * _Nullable error, NIMSignalingChannelDetailedInfo * _Nullable response) {
        NIMFChannelFullInfoResponse *ret = [[NIMFChannelFullInfoResponse alloc] initWithRequestModel:request];
        if (error) {
            ret.code = error.code;
        }
        if (response) {
            [ret setSdkResponse:response];
        }
        [NIMFSignalFlutterAPI onQueryChannelInfo:ret];
    }];
    return nil;
}

#pragma mark - Getter
- (id<NIMSignalManager>) signalManager {
    return [NIMSDK sharedSDK].signalManager;
}

@end
