//
//  NIMFSignalObserver.m
//  nim
//
//  Created by Netease on 2019/9/20.
//

#import "NIMFSignalObserver.h"
#import "NIMFSignalEvent.h"
#import "NIMFSignalFlutterAPI.h"
#import "NIMFChannelFullInfo.h"

@interface NIMFSignalObserver ()<NIMSignalManagerDelegate>

@end

@implementation NIMFSignalObserver

- (void)dealloc {
    [[NIMSDK sharedSDK].signalManager removeDelegate:self];
}

- (instancetype)init {
    if (self = [super init]) {
        [[NIMSDK sharedSDK].signalManager addDelegate:self];
    }
    return self;
}

- (NIMFSignalEvent *)eventDicWithResponse:(NIMSignalingNotifyInfo *)notifyResponse {
    NIMFSignalEvent *event = nil;
    switch (notifyResponse.eventType) {
            
        case NIMSignalingEventTypeJoin:{
            event = [[NIMFSignalJoinEvent alloc] init];
            [event setSdkResponse:notifyResponse];
            break;
        }
        case NIMSignalingEventTypeInvite:{
            event = [[NIMFSignalInviteEvent alloc] init];
            [event setSdkResponse:notifyResponse];
            break;
        }
        case NIMSignalingEventTypeCancelInvite:{
            event = [[NIMFSignalCanceledInviteEvent alloc] init];
            [event setSdkResponse:notifyResponse];
            break;
        }
        case NIMSignalingEventTypeAccept:
        case NIMSignalingEventTypeReject:
        {
            event = [[NIMFSignalInviteAckEvent alloc] init];
            [event setSdkResponse:notifyResponse];
            break;
        }
        case NIMSignalingEventTypeClose:
        case NIMSignalingEventTypeLeave:
        case NIMSignalingEventTypeContrl:
        {
            event = [[NIMFSignalEvent alloc] init];
            [event setSdkResponse:notifyResponse];
            break;
        }
        default:
            break;
    }
    return event;
}

#pragma mark - <NIMSignalManagerDelegate>

- (void)nimSignalingOnlineNotifyEventType:(NIMSignalingEventType)eventType
                                 response:(NIMSignalingNotifyInfo *)notifyResponse {
    NIMFSignalEvent *ret = [self eventDicWithResponse:notifyResponse];
    [NIMFSignalFlutterAPI onOnlineEvent:ret];
}

- (void)nimSignalingMultiClientSyncNotifyEventType:(NIMSignalingEventType)eventType
                                          response:(NIMSignalingNotifyInfo *)notifyResponse {
    NIMFSignalEvent *ret = [self eventDicWithResponse:notifyResponse];
    [NIMFSignalFlutterAPI onOtherClientEvent:ret];
}

- (void)nimSignalingOfflineNotify:(NSArray <NIMSignalingNotifyInfo *> *)notifyResponse {
    NIMFSignalOfflineEvent *ret = [[NIMFSignalOfflineEvent alloc] init];
    [ret setSdkResponse:notifyResponse];
    [NIMFSignalFlutterAPI onOfflineEvent:ret];
}

- (void)nimSignalingChannelsSyncNotify:(NSArray <NIMSignalingChannelDetailedInfo *> *)notifyResponse {
    NIMFSignalSyncEvent *event = [[NIMFSignalSyncEvent alloc] init];
    [event setSdkResponse:notifyResponse];
    [NIMFSignalFlutterAPI onSyncEvent:event];
}

- (void)nimSignalingMembersSyncNotify:(NIMSignalingChannelDetailedInfo *)notifyResponse {
    NIMFChannelFullInfoResponse * response = [[NIMFChannelFullInfoResponse alloc] init];
    [response setSdkResponse:notifyResponse];
    [NIMFSignalFlutterAPI onMemberUpdateEvent:response];
}

@end
