// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of nim;


/// auth info
class AuthInfo {
  final String? account;
  final String? token;
  String? appKey;

  AuthInfo(this.account, this.token, this.appKey);

  static AuthInfo _fromMap(Map map) {
    return AuthInfo(map['account'], map['token'], map['appKey']);
  }
  
  bool get isValid => account != null && account!.isNotEmpty;
  
  bool isMatch(account, token, appKey) => this.account == account;

  Map<String, dynamic> _toMap() =>
      <String, dynamic>{'account': account, 'token': token, 'appKey': appKey};
}

/// ios push info
class IOSPush {

  final String? apnsCername;
  final String? pkCername;

  IOSPush._(this.apnsCername, this.pkCername);

  static IOSPush _fromMap(Map map) {
    return IOSPush._(map['apnsCername'], map['pkCername']);
  }

  Map<String, dynamic> _toMap() =>
      <String, dynamic>{'apnsCername': apnsCername, 'pkCername': pkCername};
}

///android push info
class AndroidPush {

  static AndroidPush _fromMap(Map? map) {
    return AndroidPush();
  }

  Map<String, dynamic> _toMap() => <String, dynamic>{};
}

///nim options
class NIMOptions {
  IOSPush? iosPush;
  AndroidPush? androidPush;
  String? sdkDir;

  NIMOptions({this.iosPush, this.androidPush,this.sdkDir});

  static NIMOptions _fromMap(Map map) {
    return NIMOptions(
        iosPush: IOSPush._fromMap(map['iosPush']),
        androidPush: AndroidPush._fromMap(map['androidPush']),
        sdkDir:map['sdkDir']
        );
  }

  Map<String, dynamic> _toMap() =>
      <String, dynamic>{
        'iosPush':  iosPush == null ? null : iosPush!._toMap(),
        'androidPush': androidPush == null ? null : androidPush!._toMap(),
        'sdkDir': sdkDir,
      };
}

/// online client info
class OnlineClient {
  final String deviceId;
  int? clientType;
  String? os;
  int? loginTime;
  String? clientIp;
  String? consId;
  String? customTag;

  OnlineClient({required this.deviceId,
    this.clientType,
    this.os,
    this.loginTime,
    this.clientIp,
    this.consId,
    this.customTag});

  Map<String, dynamic> _toMap() {
    return <String, dynamic>{
      'deviceId': deviceId,
      'clientType': clientType,
      'os': os,
      'loginTime': loginTime,
      'clientIp': clientIp,
      'consId': consId,
      'customTag': customTag
    };
  }
}

/// auth status code
enum StatusCode {

  ///无效
  invalid,

  ///未登录/登录失败
  unLogin,

  ///被服务器拒绝连接，403，422
  forbidden,

  ///网络连接已断开
  netBroken,

  ///正在连接服务器
  connecting,

  ///正在登录中
  logining,

  ///已成功登录
  logined,

  ///客户端版本错误
  verError,

  ///用户名或密码错误
  pwdError,

  ///被其他端的登录踢掉
  kickOut,

  ///被同时在线的其他端主动踢掉
  kickByOtherClient,
}

/// auth sync status code
enum SyncStatusCode {

  ///无效
  invalid,

  ///同步开始
  syncBegin,

  ///同步完成
  syncCompletion,

  ///群组同步完成
  syncTeamCompletion,

  ///超大群组同步完成
  syncSuperTeamCompletion,
}