// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of nim;

///the auth observer
class AuthObserver extends Observable {
  @override
  String _getModule() {
    return "AuthObserver";
  }

  AuthObserver(MethodChannel methodChannel, Map<String, Handler> handlerMap) : super(methodChannel, handlerMap);

  static List<Observer<StatusCode>> onlineStatusList = <Observer<StatusCode>>[];

  ///register the observer to observe the status change
  void registerOnlineStatus(Observer<StatusCode> observer) {
    _registerObserver(observer, onlineStatusList);
  }

  ///unregister observer
  void unregisterOnlineStatus(Observer<StatusCode> observer) {
    _unregisterObserver(observer, onlineStatusList);
  }

  ///register the observer to observe the notification of kick off
  void registerKickNotify(Observer observer) {}

  @override
  Future<dynamic>? _handlerMethod(String method, Map? map) {
    switch (method) {
      case 'onOnlineStatus':
        int value = map!['statusCode'];
        StatusCode statusCode =
            value >= 0 && value < StatusCode.values.length ? StatusCode.values[value] : StatusCode.invalid;
        _notifyObserver(onlineStatusList, statusCode);
        break;
      default:
        return null;
    }
    return null;
  }
}
