// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of nim;

class AuthService extends Service {
  @override
  String _getModule() {
    return "AuthService";
  }

  AuthService(MethodChannel _methodChannel, Map<String, Handler> handlerMap) : super(_methodChannel, handlerMap);

  /// login
  Future<void> login(AuthInfo authInfo, Callback callback) async {
    Map map = buildArguments(arg: authInfo._toMap(), callback: callback);
    return await _methodChannel.invokeMethod('login', map);
  }

  /// logout
  Future<void> logout() async {
    return await _methodChannel.invokeMethod('logout', buildArguments());
  }

  /// get current account
  /// 这个方法获取的账号是 dart 层之前登录过的账号
  /// 不是实时的从nim-sdk中查询
  Future<String?> getCurrentAccount() async {
    return await _methodChannel.invokeMethod("getCurrentAccount", buildArguments());
  }

  /// if user has login
  Future<bool> isLogin() async => await getLoginInfo() != null; 

  /// 这个方法是实时从nim-sdk中获取的
  Future<AuthInfo?> getLoginInfo() async {
    final loginInfo = await _methodChannel.invokeMethod("getLoginInfo", buildArguments());
    AuthInfo? authInfo;
    if (loginInfo != null) {
      assert (loginInfo is Map);
      authInfo = AuthInfo._fromMap(loginInfo);
    }
    return authInfo != null && authInfo.isValid ? authInfo : null;
  }

  ///kick off other client
  Future<void> kickOtherClient(OnlineClient onlineClient, Callback callback) async {
    Map map = buildArguments(arg: onlineClient._toMap(), callback: callback);
    return await _methodChannel.invokeMethod("kickOtherClient", map);
  }

  @override
  Future<dynamic>? _handlerMethod(String method, int code, Map arg, dynamic callback) {
    switch (method) {
      case "onLogin":
      case "onKickOtherClient":
        callback(code, null);
        break;
    }
    return null;
  }
}
