// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of nim;

typedef Observer<T> = void Function(T data);

typedef Callback<T> = void Function(int code, T? data);

/// base observable
abstract class Observable extends Handler {
  final MethodChannel _methodChannel;

  Observable(this._methodChannel, Map<String, Handler> handlerMap) : super(handlerMap);

  Future<dynamic>? _handlerMethod(String method, Map? arg);

  @override
  Future? _handler(MethodCall call) {
    return _handlerMethod(call.method, call.arguments);
  }

  /// register observer type
  void _registerObserver<T>(Observer<T> observer, List<Observer<T>> list) {
    if (list.contains(observer)) {
      return;
    }
    list.add(observer);
  }

  /// unregister observer T type
  void _unregisterObserver<T>(Observer<T> observer, List<Observer<T>> list) {
    if (list.contains(observer)) {
      list.remove(observer);
    }
  }

  /// notify
  void _notifyObserver<T>(List<Observer<T>> list, T data) {
    if (list.isEmpty || data == null) {
      return;
    }
    for (Observer<T> observer in list) {
      observer(data);
    }
  }
}
