// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
part of nim;

abstract class Service extends Handler {

  final MethodChannel _methodChannel;

  final String receiptIdKey = "receiptId";

  Service(this._methodChannel, Map<String, Handler> handlerMap) :super(handlerMap);

  static int initId = 0;

  /// native 2 flutter callback
  Future<dynamic>? _handlerMethod(String method, int code, Map arg, dynamic callback);

  // inner use 
  String _generateReceiptId() {
    initId++;
    return _getModule() + initId.toString();
  }

  Map<String, dynamic> callbacks = Map<String, dynamic>();

  ///build arguments for methodChannel
  Map<String, dynamic> buildArguments<T>({Map? arg, Callback<T>? callback}) {
    var map = <String, dynamic>{'module': _getModule()};
    if (callback != null) {
      var receiptId = _generateReceiptId();
      callbacks[receiptId] = callback;
      map[receiptIdKey] = receiptId;
    }
    if (arg != null && arg.isNotEmpty) {
      map.addAll(arg as Map<String, dynamic>);
    }
    return map;
  }

  dynamic _queryCallback({required Map map, bool oneShot = true}) {
    String receiptId = map[receiptIdKey];
    print("_queryCallback key=" + receiptId);
    if (callbacks.containsKey(receiptId)) {
      return callbacks.remove(receiptId);
    }
    return null;
  }

  @override
  Future? _handler(MethodCall call) {
    Map arg = call.arguments;
    dynamic callback = _queryCallback(map: arg);
    if (callback == null) {
      return null;
    }
    int code = call.arguments['code'] ?? 0;
    return _handlerMethod(call.method, code, arg, callback);
  }

}
