// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class ChatRoomMessageBuilder {

  ///create message by id
  static ChatRoomMessage _initSendMessage(String toId) {
    ChatRoomMessage message = ChatRoomMessage();
    message.sessionId = toId;
    return message;
  }

  ///create message by id and content
  static ChatRoomMessage createTextMessage(String sessionId, String content) {
    ChatRoomMessage message = _initSendMessage(sessionId);
    message.msgType = MsgType.text;
    message.content = content;
    return message;
  }

  ///create message by id and attachment
  static ChatRoomMessage createImageMessage(String sessionId, String filePath, String displayName) {
    return createImageMessage1(sessionId, filePath, displayName, NosSceneKeyConstant.nimDefaultIM);
  }

  ///create message by id and attachment
  static ChatRoomMessage createImageMessage1(
      String sessionId, String filePath, String displayName, String nosTokenSceneKey) {
    ChatRoomMessage message = _initSendMessage(sessionId);
    message.msgType = MsgType.image;
    ImageAttachment attachment = new ImageAttachment();
    attachment.path = filePath;
    attachment.displayName = displayName;
    attachment.nosTokenSceneKey = nosTokenSceneKey;
    message.attachment = attachment;
    return message;
  }

  static ChatRoomMessage createFileMessage(String sessionId, String filePath, String displayName) {
    return createFileMessage1(sessionId, filePath, displayName, NosSceneKeyConstant.nimDefaultIM);
  }

  static ChatRoomMessage createFileMessage1(
      String sessionId, String filePath, String displayName, String nosTokenSceneKey) {
    ChatRoomMessage message = _initSendMessage(sessionId);
    message.msgType = MsgType.file;
    FileAttachment attachment = new FileAttachment();
    attachment.path = filePath;
    if(filePath.length > 0) {
      File file = File(filePath);
      if(file.existsSync()) {
        attachment.size = file.lengthSync();
      }
    }
    attachment.displayName = displayName;
    attachment.nosTokenSceneKey = nosTokenSceneKey;
    message.attachment = attachment;
    return message;
  }
}
