// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

/// 性别
enum Gender {
  /// 未知性别
  unknown,

  /// 性别男
  male,

  /// 性别女
  female,
}

/// 操作类型
enum FriendOperation {
  /// 直接添加好友，无需验证
  add,

  /// 请求添加好友
  request,

  /// 通过添加好友请求
  pass,

  /// 拒绝添加好友请求
  reject,
}


/// 云信好友
class User {
  /// 用户ID
  final String? userId;

  /// 备注名，长度限制为128个字符
  String? alias;

  /// 扩展字段
  String? ext;

  /// 服务端扩展字段
  String? serverExt;

  /// 是否需要消息提醒
  bool? notifyForNewMsg;

  /// 是否在黑名单
  bool? isInBlackList;

  /// 用户资料
  UserInfo? userInfo;

  User({
    required this.userId,
    this.alias,
    this.ext,
    this.serverExt,
    this.notifyForNewMsg,
    this.isInBlackList,
    this.userInfo,
  });

  static User? fromMap(Map? map) {
    if (map == null) {
      return null;
    }

    Map userInfoMap = map['userInfo'];

    return User(
      userId: map['userId'],
      alias: map['alias'],
      serverExt: map['serverExt'],
      notifyForNewMsg: map['notifyForNewMsg'],
      isInBlackList: map['isInBlackList'],
      userInfo: UserInfo.fromMap(userInfoMap)
    );
  }

}

/// 用户资料信息
class UserInfo {
  /// 用户昵称
  final String? nickName;

  /// 用户头像
  final String? avatarUrl;

  /// 用户头像缩略图
  final String? thumbAvatarUrl;

  /// 用户签名
  final String? sign;

  /// 用户性别 [Gender]
  final Gender? gender;

  /// 邮箱
  final String? email;

  /// 生日
  final String? birth;

  /// 电话号码
  final String? mobile;

  /// 用户自定义扩展字段
  final String? ext;

  UserInfo({
    this.nickName,
    this.avatarUrl,
    this.thumbAvatarUrl,
    this.sign,
    this.gender,
    this.email,
    this.birth,
    this.mobile,
    this.ext,
  });

  static UserInfo fromMap(Map map) {
    return UserInfo(
      nickName: map['nickName'],
      avatarUrl: map['avatarUrl'],
      thumbAvatarUrl: map['thumbAvatarUrl'],
      sign: map['sign'],
      gender: map['gender'] == 0 ? Gender.unknown : ((map['gender'] == 1) ? Gender.male : Gender.female),
      email: map['email'],
      birth: map['birth'],
      mobile: map['mobile'],
      ext: map['ext'],
    );
  }
}

/// 好友请求
class FriendRequest {
  /// 目标用户
  String? userId;

  /// 操作类型
  FriendOperation? operation;

  /// 附言
  String? attach;

  FriendRequest(
    this.userId,
    this.operation,
    this.attach
  );

  static FriendRequest _fromMap(Map map) {
    return FriendRequest(map['userId'], map['operation'], map['attach']);
  }

  Map<String, dynamic> _toMap() =>
      <String, dynamic>{'userId': userId, 'operation': operation, 'attach': attach};
}