// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

/// session type
class SessionType {
  static const int p2p = 0;
  static const int team = 1;
  static const int superTeam = 5;
  static const int system = 10001;
  static const int chatRoom = 10002;
}

/// msg type
class MsgType {
  static const int undef = -1;

  static const int text = 0;

  static const int image = 1;

  static const int audio = 2;

  static const int video = 3;

  static const int location = 4;

  static const int notification = 5;

  static const int file = 6;

  static const int avchat = 7;

  static const int vchat = 8; // hold

  static const int tip = 10; //提醒类型消息

  static const int robot = 11; //机器人消息

  static const int custom = 100;
}

/// msg status
class MsgStatus {
  static const int draft = -1;
  static const int sending = 0;
  static const int success = 1;
  static const int fail = 2;
  static const int read = 3;
  static const int unread = 4;
}

class MsgDirection {
  static const int out = 0;
  static const int receive = 1;
}

class AttachStatus {
  static const int def = 0;
  static const int transferring = 1;
  static const int transferred = 2;
  static const int fail = 3;
  static const int cancel = 4;
}

class CustomMessageConfig {}

class PushOptions {}

class AntiOptions {}

class IMMessage {
  String? uuid;

  String? sessionId;

  int? sessionType;

  int? msgType;

  int? msgStatus;

  int? msgDirection;

  String? content;

  int? time;

  String? fromAccount;

  String? attach;

  int? attachStatus;

  CustomMessageConfig? config;

  String? remoteExt;

  String? localExt;

  String? pushContent;

  String? pushPayload;

  PushOptions? pushOptions;

  int? serverId;

  AntiOptions? antiOptions;

  bool? msgAck;

  bool? hasSendAck;

  bool? ackCount;

  bool? unAckCount;

  MsgAttachment? attachment;

  static IMMessage _fromMap(var map) {
    IMMessage message = IMMessage();
    _parserMap(message, map);
    return message;
  }

  static void _parserMap(IMMessage message, var map) {
    message.uuid = map['uuid'];
    message.sessionId = map['sessionId'];
    message.sessionType = map['sessionType'];
    message.attach = map['attach'];
    message.time = map['time'];
    message.serverId = map['serverId'];
    message.msgType = map['msgType'];
    message.content = map['content'];
    message.fromAccount = map['fromAccount'];
    message.msgStatus = map['msgStatus'];
    message.msgDirection = map['msgDirection'];
    message.attachment = AttachFactory().parseAttach(message.msgType, message.attach);
  }

  Map<String, dynamic> _toMap() => {
        'sessionId': sessionId,
        'sessionType': sessionType,
        'attach': attachment?.toJson(),
        'msgType': msgType,
        'time': time,
        'serverId': serverId,
        'content': content,
        'fromAccount': fromAccount,
        'msgStatus': msgStatus,
        'msgDirection': msgDirection
      };
}
