// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class NimPlugin {
  static NimPlugin? _instance;

  factory NimPlugin() {
    if (_instance == null) {
      final MethodChannel _channel = const MethodChannel('nim_plugin');
      _instance = NimPlugin.private(_channel);
    }
    return _instance!;
  }

  final MethodChannel _methodChannel;

  static Map<String, Handler> handlerMap = Map();

  static AuthService? _authService;

  static AuthObserver? _authObserver;

  static MsgService? _msgService;

  static MsgObserver? _msgObserver;

  static SignallingService? _signallingService;

  static SignallingObserver? _signallingObserver;

  static UserService? _friendService;

  static UserObserver? _friendObserver;

  static PassthroughService? _passthroughService;

  static PassthroughObserver? _passthroughObserver;

  static ResourceService? _resourceService;

  static ResourceObserver? _resourceObserver;

  static ChatRoomService? _chatRoomService;

  static ChatRoomObserver? _chatRoomObserver;

  NimPlugin.private(this._methodChannel) {
    _methodChannel.setMethodCallHandler(_handler);
  }

  /// config & init
  Future<void> config({required String? appKey, AuthInfo? authInfo, NIMOptions? options}) async {
    return await _methodChannel
        .invokeMethod("config", {'appKey': appKey, 'authInfo': authInfo?._toMap(), 'options': options?._toMap()});
  }

  /// Auth service
  AuthService getAuthService() {
    return _authService ?? (_authService = AuthService(_methodChannel, handlerMap));
  }

  /// Auth observer
  AuthObserver getAuthObserver() {
    return _authObserver ?? (_authObserver = AuthObserver(_methodChannel, handlerMap));
  }

  /// Msg service
  MsgService getMsgService() {
    return _msgService ?? (_msgService = MsgService(_methodChannel, handlerMap));
  }

  /// Msg observer
  MsgObserver getMsgObserver() {
    return _msgObserver ?? (_msgObserver = MsgObserver(_methodChannel, handlerMap));
  }

  /// Signalling service
  SignallingService getSignallingService() {
    return _signallingService ?? (_signallingService = SignallingService(_methodChannel, handlerMap));
  }

  /// Signalling observer
  SignallingObserver getSignallingObserver() {
    return _signallingObserver ?? (_signallingObserver = SignallingObserver(_methodChannel, handlerMap));
  }

  /// Passthrough service
  PassthroughService getPassthroughService() {
    return _passthroughService ?? (_passthroughService = PassthroughService(_methodChannel, handlerMap));
  }

  PassthroughObserver getPassthroughObserver() {
    return _passthroughObserver ?? (_passthroughObserver = PassthroughObserver(_methodChannel, handlerMap));
  }

  /// Friend Service
  UserService getFriendService() {
    return _friendService ?? (_friendService = UserService(_methodChannel, handlerMap));
  }

  /// Friend Observer
  UserObserver getFriendObserver() {
    return _friendObserver ?? (_friendObserver = UserObserver(_methodChannel, handlerMap));
  }

  /// Resource Service
  ResourceService getResourceService() {
    return _resourceService ?? (_resourceService = ResourceService(_methodChannel, handlerMap));
  }

  /// Resource Observer
  ResourceObserver getResourceObserver() {
    return _resourceObserver ?? (_resourceObserver = ResourceObserver(_methodChannel, handlerMap));
  }

  /// ChatRoom Service
  ChatRoomService getChatRoomService() {
    return _chatRoomService ?? (_chatRoomService = ChatRoomService(_methodChannel, handlerMap));
  }

  /// ChatRoom Observer
  ChatRoomObserver getChatRoomObserver() {
    return _chatRoomObserver ?? (_chatRoomObserver = ChatRoomObserver(_methodChannel, handlerMap));
  }

  /// native  --  dart
  Future<dynamic> _handler(MethodCall call) {
    Map? map = call.arguments;
    if (map == null) {
      return Future.value(null);
    }
    String module = map['module'] ?? "";
    Handler? handler = handlerMap[module];
    print("_handler method= " + call.method + " module=" + module + "handler=" + handler.toString());
    return handler?._handler(call) ?? Future.value(null);
  }
}
