// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class SceneKey {
  /// （用户、群组..资料（eg:头像）的上传默认走这个场景）
  static final String _nimDefaultProfile = "nim_default_profile_icon";

  ///（私聊、群聊、聊天室发送图片、音频、视频、文件..默认走这个场景）
  static final String _nimDefaultIm = "nim_default_im";

  ///  （sdk内部上传文件（eg:日志）默认走这个场景）并且对应的过期时间不允许修改
  static final String _nimSystemNosScene = "nim_system_nos_scene";

  /// ios scene key
  /// （用户、群组..资料（eg:头像）的上传默认走这个场景）
  static final String _iosNimDefaultProfile = "nim_icon";

  ///（私聊、群聊、聊天室发送图片、音频、视频、文件..默认走这个场景）
  static final String _iosNimDefaultIm = "nim_msg";

  ///  （sdk内部上传文件（eg:日志）默认走这个场景）并且对应的过期时间不允许修改
  static final String _iosNimSystemNosScene = "nim_system";

  static bool _isIOSPlatform() {
    return defaultTargetPlatform == TargetPlatform.iOS;
  }

  /// （用户、群组..资料（eg:头像）的上传默认走这个场景）
  static String get nimDefaultProfile{
    return _isIOSPlatform() ? _iosNimDefaultProfile : _nimDefaultProfile;
  }

  ///（私聊、群聊、聊天室发送图片、音频、视频、文件..默认走这个场景）
  static String get nimDefaultIm{
    return _isIOSPlatform() ? _iosNimDefaultIm : _nimDefaultIm;
  }

  ///  （sdk内部上传文件（eg:日志）默认走这个场景）并且对应的过期时间不允许修改
  static String get nimSystemNosScene{
    return _isIOSPlatform() ? _iosNimSystemNosScene : _nimSystemNosScene;
  }
}

enum TransferType {
  /// upload
  upload,

  /// download
  download,

  /// error
  error
}

enum NosTransferStatus {
  /// def default
  def,

  /// transferring
  transferring,

  /// transferred
  transferred,

  /// fail
  fail
}

class NosTransferInfo {
  /// upload or download
  TransferType? transferType;

  /// file path
  String? path;

  /// file size
  String? size;

  /// file content md5
  String? md5;

  /// file download url
  String? url;

  /// file ext
  String? extension;

  /// file transfer status
  NosTransferStatus? status;

  static NosTransferInfo _fromMap(Map arg) {
    NosTransferInfo info = NosTransferInfo();
    int type = arg['transferType'];
    info.transferType =
        (type >= TransferType.values.length || type < 0) ? TransferType.error : TransferType.values[type];
    info.path = arg['path'];
    info.size = arg['size'];
    info.md5 = arg['md5'];
    info.url = arg['url'];
    info.extension = arg['extension'];
    int status = arg['status'];
    info.status = (status >= 0 && status < NosTransferStatus.values.length)
        ? NosTransferStatus.values[status]
        : NosTransferStatus.def;
    return info;
  }
}

class NosTransferProgress {
  /// 获取文件的唯一标识（上传时key是文件路径，下载时key为URL）
  String? key;

  /// 传输进度
  double? progress;

  static NosTransferProgress _fromMap(Map arg) {
    NosTransferProgress progress = NosTransferProgress();
    progress.key = arg['key'];
    progress.progress = arg['progress'];
    return progress;
  }
}
