// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class ResourceService extends Service {
  ResourceService(MethodChannel _methodChannel, Map<String, Handler> handlerMap) : super(_methodChannel, handlerMap);

  @override
  String _getModule() {
    return 'ResourceService';
  }

  /// upload file to nos
  /// filePath file path
  /// mimeType file mimeType
  /// sceneKey file upload scene  @see : SceneKey
  /// forceUpload  如果服务器存在相同的文件，是否强制重新上传文件 ，默认false
  Future<void> uploadAtScene(
      {required String filePath, String? mimeType, String? sceneKey, bool? forceUpload, Callback<String>? callback}) async {
    Map map = buildArguments(callback: callback);
    map['filePath'] = filePath;
    map['mimeType'] = mimeType;
    map['sceneKey'] = sceneKey;
    map['forceUpload'] = forceUpload;
    return await _methodChannel.invokeMethod("uploadAtScene", map);
  }

  /// 针对开启了NOS 文件安全功能的用户，可能通过短链获取原始链接，注意保护原始链接不要外泄
  /// roomId 聊天室id
  /// shortUrl 短链
  Future<void> getOriginUrlFromShortUrl({String? roomId, required String shortUrl, Callback<String>? callback}) async {
    Map map = buildArguments(callback: callback);
    map['roomId'] = roomId;
    map['shortUrl'] = shortUrl;
    return await _methodChannel.invokeMethod("getOriginUrlFromShortUrl", map);
  }

  /// 将文件url 转换成CDN 加速url ，如果不支持，返回原链接
  /// roomId 聊天室id
  /// shortUrl 短链
  Future<void> convertDownloadUrlToCDNUrl({required String url, Callback<String>? callback}) async {
    Map map = buildArguments(callback: callback);
    map['url'] = url;
    return await _methodChannel.invokeMethod("convertDownloadUrlToCDNUrl", map);
  }

  Future<dynamic>? _handlerMethod(String method, int code, Map? arg, dynamic callback) {
    switch (method) {
      case "onUploadAtScene":
      case "onConvertDownloadUrlToCDNUrl":
      case "onGetOriginUrlFromShortUrl":
        String? url = arg != null ? arg['url'] : null;
        callback(code, url);
        break;
    }
    return null;
  }
}
