// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

part of nim;

class SignallingObserver extends Observable {
  SignallingObserver(MethodChannel methodChannel, handlerMap) : super(methodChannel, handlerMap);

  @override
  String _getModule() {
    return "SignallingObserver";
  }

  static List<Observer<SignallingEvent>> onlineEvent = <Observer<SignallingEvent>>[];

  static List<Observer<List<SignallingEvent>?>> offlineEvent = <Observer<List<SignallingEvent>?>>[];

  static List<Observer<InviteAckEvent>> otherClientEvent = <Observer<InviteAckEvent>>[];

  static List<Observer<List<ChannelFullInfo?>?>> syncEvent = <Observer<List<ChannelFullInfo?>?>>[];

  static List<Observer<ChannelFullInfo?>> memberUpdateEvent = <Observer<ChannelFullInfo?>>[];

  /// 注册 在线通知事件观察者
  void registerOnlineEvent(Observer<SignallingEvent> observer) {
    _registerObserver(observer, onlineEvent);
  }

  ///注销 在线通知事件观察者
  void unregisterOnlineEvent(Observer<SignallingEvent> observer) {
    _unregisterObserver(observer, onlineEvent);
  }

  ///注册 离线通知事件观察者，在用户登录后sdk 会去服务器同步用户离线期间发生的各种通知，并以列表的形式返回。列表的排序方式为时间递增排序
  void registerOfflineEvent(Observer<List<SignallingEvent>> observer) {
    _registerObserver(observer, offlineEvent);
  }

  /// 注销 离线通知事件观察者
  void unregisterOfflineEvent(Observer<List<SignallingEvent>> observer) {
    _unregisterObserver(observer, offlineEvent);
  }

  /// 注册 其他端响应（接收/拒绝）邀请事件观察者，当其他端响应了邀请时触发
  void registerOtherClientEvent(Observer<InviteAckEvent> observer) {
    _registerObserver(observer, otherClientEvent);
  }

  ///注销 其他端响应（接收/拒绝）邀请事件观察者，当其他端响应了邀请时触发
  void unregisterOtherClientEvent(Observer<InviteAckEvent> observer) {
    _unregisterObserver(observer, otherClientEvent);
  }

  ///注册 同步未退出频道列表事件观察者 ，在用户登录后sdk会去服务器获取当前还未退出的频道列表
  void registerSyncEvent(Observer<List<ChannelFullInfo>> observer) {
    _registerObserver(observer, syncEvent);
  }

  /// 注销 同步未退出频道列表事件观察者 ，在用户登录后sdk会去服务器获取当前还未退出的频道列表
  void unregisterSyncEvent(Observer<List<ChannelFullInfo>> observer) {
    _unregisterObserver(observer, syncEvent);
  }

  /// 注册 频道成员更新事件观察者 如果有成员信息的变更（而不是成员数量增减）或异常情况
  void registerMemberUpdateEvent(Observer<ChannelFullInfo> observer) {
    _registerObserver(observer, memberUpdateEvent);
  }

  ///注销 频道成员更新事件观察者
  void unregisterMemberUpdateEvent(Observer<ChannelFullInfo> observer) {
    _unregisterObserver(observer, memberUpdateEvent);
  }

  @override
  Future? _handlerMethod(String method, Map? map) {
    switch (method) {
      case 'onOnlineEvent':
        SignallingEvent event = EventFactory._fromMap(map!);
        _notifyObserver(onlineEvent, event);
        break;
      case 'onOfflineEvent':
        List? events = map!['events'];
        List<SignallingEvent>? list =
            events != null ? events.map<SignallingEvent>((i) => EventFactory._fromMap(i)).toList() : null;
        _notifyObserver(offlineEvent, list);
        break;
      case 'onOtherClientEvent':
        InviteAckEvent event = InviteAckEvent.__fromMap(map!);
        _notifyObserver(otherClientEvent, event);
        break;
      case 'onSyncEvent':
        List? events = map!['events'];
        List<ChannelFullInfo?>? list =
            events != null ? events.map<ChannelFullInfo?>((i) => ChannelFullInfo._fromMap(i)).toList() : null;
        _notifyObserver(syncEvent, list);
        break;
      case 'onMemberUpdateEvent':
        ChannelFullInfo? fullInfo = ChannelFullInfo._fromMap(map);
        _notifyObserver(memberUpdateEvent, fullInfo);
        break;
      default:
        return null;
    }
    return null;
  }
}
